# Generated by Django 2.2.4 on 2019-11-05 17:04

from django.db import migrations, models
import djmongo.dataimport.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DataImport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file1', models.FileField(upload_to=djmongo.dataimport.models.update_import_filename, verbose_name='File to be Imported')),
                ('input_format', models.CharField(choices=[('csv', 'Comma Separated Value (.csv)')], default='csv', max_length=3)),
                ('delete_collection_before_import', models.BooleanField(default=False)),
                ('status', models.CharField(default='New', editable=False, max_length=10, verbose_name='Status')),
                ('response', models.TextField(blank=True, default='', max_length=2048, verbose_name='Response')),
                ('database_name', models.CharField(max_length=256)),
                ('collection_name', models.CharField(max_length=256)),
                ('creation_date', models.DateField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
            },
        ),
    ]



# Generated by Django 2.2.4 on 2019-11-05 17:04

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='WriteAPIOAuth2',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('scopes', models.CharField(blank=True, default='*', help_text='Space delimited list of scopes required. * means no scope is required.', max_length=1024)),
                ('http_post', models.BooleanField(blank=True, default=True)),
                ('http_put', models.BooleanField(blank=True, default=True)),
                ('slug', models.SlugField(max_length=100, unique=True)),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('json_schema', models.TextField(default='{}', help_text='Default "{}", means no JSON Schema.', max_length=20480, verbose_name='JSON Schema')),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Write API with OAuth2 Auth',
                'verbose_name_plural': 'Write APIs with OAuth2 Auth',
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
            },
        ),
        migrations.CreateModel(
            name='WriteAPIIP',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(max_length=100, unique=True)),
                ('http_post', models.BooleanField(blank=True, default=True)),
                ('http_put', models.BooleanField(blank=True, default=True)),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('json_schema', models.TextField(default='{}', help_text='Default "{}", means no JSON Schema.', max_length=20480, verbose_name='JSON Schema')),
                ('from_ip', models.TextField(default='127.0.0.1', help_text='Only accept requests from a IP in this list separated by whitespace . 0.0.0.0 means all.', max_length=2048, verbose_name='From IPs')),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Write API with IP Address Auth',
                'verbose_name_plural': 'Write APIs with IP Address Auth',
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
            },
        ),
        migrations.CreateModel(
            name='WriteAPIHTTPAuth',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(help_text='The slug is the unique part of the URL for your API.', max_length=100, unique=True)),
                ('http_post', models.BooleanField(blank=True, default=True)),
                ('http_put', models.BooleanField(blank=True, default=True)),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('json_schema', models.TextField(default='{}', help_text='Default "{}", means no JSON Schema.', max_length=20480, verbose_name='JSON Schema')),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
                ('groups', models.ManyToManyField(blank=True, help_text='Use ctrl to select multiple groups. If no groups are selected blank, any user may access the API.', related_name='djmongo_write_httpauth_groups', to='auth.Group')),
            ],
            options={
                'verbose_name': 'Write API with HTTPAuth',
                'verbose_name_plural': 'Write APIs with HTTPAuth',
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
            },
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.shortcuts import render, get_object_or_404
from django.http import HttpResponseRedirect
from django.contrib import messages
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from .forms import DataImportForm
from .models import DataImport


def delete_import(request, slug):
    im = get_object_or_404(DataImport, slug=slug)
    im.delete()
    messages.success(request,
                     _("The record of the import was deleted. "
                       "No data was not removed from MongoDB."))
    return HttpResponseRedirect(reverse('show_dbs'))


def import_data_file(request, database_name=None, collection_name=None):
    name = _("Import CSV File")

    if request.method == 'POST':
        form = DataImportForm(request.POST, request.FILES)
        if form.is_valid():
            di = form.save()
            messages.success(request, _("The data was imported successfully."))
            return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})

    # this is a GET
    idata = {}
    if database_name:
        idata['database_name'] = database_name
    if collection_name:
        idata['collection_name'] = collection_name
    messages.warning(
        request,
        _("""For large data imports use the command line utility csv2mongo.
          Its part of "JSON Data Tools" and can be installed with "pip install
          jdt"."""))
    context = {'name': name,
               'form': DataImportForm(initial=idata),
               }
    return render(request,
                  'djmongo/console/generic/bootstrapform.html',
                  context)


def previous_data_imports(request, database_name=None, collection_name=None):

    if not database_name or collection_name:
        dataimports = DataImport.objects.filter(database_name=database_name,
                                                collection_name=database_name)
    else:
        dataimports = DataImport.objects.all()

    context = {"dataimports": dataimports,
               "database_name": database_name,
               "collection_name": collection_name}
    return render(request,
                  'djmongo/console/previous-imports.html',
                  context)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.core.management.base import NoArgsCommand
import datetime
import calendar
import json
from djmongo.aggregations.models import Aggregation
from djmongo.mongoutils import run_aggregation_pipeline


class Command(NoArgsCommand):
    help = 'Check for aggregate jobs that have not run. ' \
           'Add this to your crontab to execute every 5 ' \
           'minutes or so.'

    def handle_noargs(self, **options):
        now = datetime.datetime.now()
        now_time = datetime.datetime.time(datetime.datetime.now())
        out = "It is now %s on %s " % (
            now_time, calendar.day_name[now.weekday()])
        weekday = calendar.day_name[now.weekday()]
        self.stdout.write(out)

        if weekday == "Sunday":
            aggs = Aggregation.objects.filter(
                execute_sunday=True,
                execute_time_1__lte=now_time,
                scheduled_job_in_process=False,
                scheduled_job_completed_for_today=False)
        if weekday == "Monday":
            aggs = Aggregation.objects.filter(
                execute_monday=True,
                execute_time_1__lte=now_time,
                scheduled_job_in_process=False,
                scheduled_job_completed_for_today=False)
        if weekday == "Tuesday":
            aggs = Aggregation.objects.filter(
                execute_tuesday=True,
                execute_time_1__lte=now_time,
                scheduled_job_in_process=False,
                scheduled_job_completed_for_today=False)
        if weekday == "Wednesday":
            aggs = Aggregation.objects.filter(
                execute_wednesday=True,
                execute_time_1__lte=now_time,
                scheduled_job_in_process=False,
                scheduled_job_completed_for_today=False)
        if weekday == "Thursday":
            aggs = Aggregation.objects.filter(
                execute_thursday=True,
                execute_time_1__lte=now_time,
                scheduled_job_in_process=False,
                scheduled_job_completed_for_today=False)
        if weekday == "Friday":
            aggs = Aggregation.objects.filter(
                execute_friday=True,
                execute_time_1__lte=now_time,
                scheduled_job_in_process=False,
                scheduled_job_completed_for_today=False)
        if weekday == "Saturday":
            aggs = Aggregation.objects.filter(
                execute_saturday=True,
                execute_time_1__lte=now_time,
                scheduled_job_in_process=False,
                scheduled_job_completed_for_today=False)

        if aggs.count() == 0:
            print("No jobs to run right now")

        for a in aggs:
            msg = "Run job %s" % (a.slug)

            a.scheduled_job_in_process = True
            a.save()
            self.stdout.write(msg)

            # Perform the aggregation
            pipeline = json.loads(a.pipeline)
            output_dict = {"$out": a.output_collection_name}
            pipeline.append(output_dict)
            run_aggregation_pipeline(a.database_name, a.collection_name,
                                     pipeline)

            # Mark it as complete and no longer in process.
            a.scheduled_job_completed_for_today = True
            a.scheduled_job_in_process = False
            a.save()
            msg = "Job %s completed." % (a.slug)
            print(msg)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4


"""
    Decorator to check for credentials before responding on json requests.
"""

from functools import update_wrapper, wraps
from django.http import HttpResponse
from django.contrib.auth import authenticate, login
from .utils import unauthorized_json_response, user_permissions


def json_login_required(func):
    """
        Put this decorator before your view to check if the user is logged in
        and return a JSON 401 error if he/she is not.
    """

    def wrapper(request, *args, **kwargs):
        user = None
        # get the Basic username and password from the request.
        auth_string = request.META.get('HTTP_AUTHORIZATION', None)

        if auth_string:
            (authmeth, auth) = auth_string.split(" ", 1)
            auth = auth.strip().decode('base64')
            (username, password) = auth.split(':', 1)

            # print username, password
            user = authenticate(username=username, password=password)

        if not user or not user.is_active:
            return HttpResponse(unauthorized_json_response(), status=401,
                                content_type="application/json")
        login(request, user)
        return func(request, *args, **kwargs)

    return update_wrapper(wrapper, func)


def access_required(permission):
    def decorator(func):
        def inner_decorator(request, *args, **kwargs):
            if permission in user_permissions(request):
                return func(request, *args, **kwargs)
            else:
                return HttpResponse(unauthorized_json_response(), status=401,
                                    content_type="application/json")
        return wraps(func)(inner_decorator)

    return decorator



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

import json
from django import forms
from .models import Aggregation
from django.utils.translation import ugettext_lazy as _


class AggregationForm(forms.ModelForm):

    class Meta:
        model = Aggregation
        fields = ('slug', 'pipeline', 'database_name', 'collection_name',
                  'output_collection_name', 'execute_time_1', 'description')

    required_css_class = 'required'

    def clean_pipeline(self):
        pipeline = self.cleaned_data.get('pipeline')

        try:
            j = json.loads(pipeline)
            if not isinstance(j, type([])):
                msg = _("The pipeline is not an array/list.")
                raise forms.ValidationError(msg)
            else:
                for i in j:
                    if not isinstance(i, type({})):
                        msg = _("The pipeline is not an array/list of dicts.")
                        raise forms.ValidationError(msg)
                    if "$out" in i:
                        msg = _("""Do not provide $out in your pipeline.
                                Use output_collection_name instead.""")
                        raise forms.ValidationError(msg)

        except ValueError:
            msg = _("The pipeline field does not contain valid JSON.")
            raise forms.ValidationError(msg)

        return pipeline



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from ..decorators import httpauth_login_required
from django.conf.urls import url
from .views import (showdbs, create_new_database,
                    create_collection, drop_collection,
                    simple_ensure_index, create_document_in_collection,
                    update_document_in_collection, drop_database,
                    show_apis, api_wizard, api_list)

urlpatterns = [

    url(r'^$', login_required(showdbs), name="djmongo_show_dbs"),

    url(r'^show-apis/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)',
        login_required(show_apis), name="djmongo_show_apis"),


    url(r'^api-list',
        login_required(api_list), name="djmongo_api_list"),

    url(r'^api-wizard/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)',
        login_required(api_wizard), name="djmongo_api_wizard_w_params"),

    url(r'^api-wizard',
        login_required(api_wizard), name="djmongo_api_wizard"),

    url(r'^new-database$', create_new_database,
        name="djmongo_create_new_database"),

    url(r'^drop-collection/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)',
        login_required(drop_collection), name="djmongo_drop_collection"),

    url(r'^ensure-index/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(simple_ensure_index), name="djmongo_ensure_index"),

    url(r'create-document/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(create_document_in_collection),
        name="djmongo_create_document_in_collection_w_params"),

    url(r'^update-document/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(update_document_in_collection),
        name="djmongo_update_document_in_collection_w_params"),

    url(r'^drop-database/(?P<database_name>[^/]+)$',
        login_required(drop_database), name="djmongo_drop_database"),

    url(r'^create-collection/(?P<database_name>[^/]+)$',
        login_required(create_collection), name="djmongo_create_collection"),

    # API calls --------------------------------------------------------------

    url(r'^api/drop-collection/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        httpauth_login_required(drop_collection),
        name="djmongo_api_delete_collection"),

    url(r'^api/ensure-index/(?P<database_name>[^/]+)/(?P<collection_name>\[^/]+)$',
        httpauth_login_required(csrf_exempt(simple_ensure_index)),
        name="djmongo_api_djmongo_ensure_indexe"),

    url(r'^api/drop-database/(?P<database_name>[^/]+)$',
        httpauth_login_required(drop_database),
        name="djmongo_api_drop_database"),

    url(r'^api/create-collection/(?P<database_name>[^/]+)$',
        httpauth_login_required(create_collection),
        name="djmongo_api_create_collection"),

]



from django.db import models
from django.conf import settings
from django.contrib.auth.models import Group
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse

OUTPUT_CHOICES = (("json", "JSON"),
                  ("html", "HTML"),
                  ("csv", "Comma Separated Value (.csv)"))


@python_2_unicode_compatible
class CustomOAuth2ReadAPI(models.Model):
    output_format = models.CharField(max_length=4,
                                     choices=OUTPUT_CHOICES,
                                     default="json")
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")

    scopes = models.CharField(max_length=1024, default="*", blank=True,
                              help_text="Space delimited list of scopes required. * means no scope is required.")
    query = models.TextField(max_length=2048, default="{}",
                             verbose_name="JSON Query")
    type_mapper = models.TextField(
        max_length=2048,
        default="{}",
        verbose_name=_("Map non-string variables to numbers or Boolean"))
    sort = models.TextField(
        max_length=2048,
        default="",
        blank=True,
        verbose_name="Sort Dict",
        help_text="""e.g. [["somefield", 1], ["someotherfield", -1] ]""")

    return_keys = models.TextField(max_length=2048, default="", blank=True,
                                   help_text=_("Default is blank which "
                                               "returns all keys. Separate "
                                               "keys by white space to limit"
                                               "the keys that are returned."))

    default_limit = models.IntegerField(
        default=getattr(
            settings,
            'MONGO_LIMIT',
            200),
        help_text="Limit results to this number unless specified otherwise.",
    )
    database_name = models.CharField(max_length=100)
    collection_name = models.CharField(max_length=100)
    creation_date = models.DateField(auto_now_add=True)
    readme_md = models.TextField(max_length=4096, default="", blank=True)

    class Meta:
        get_latest_by = "creation_date"
        ordering = ('-creation_date',)
        unique_together = (('database_name', 'collection_name', 'slug'), )

    def __str__(self):
        return "%s/%s/%s" % (self.database_name, self.collection_name, self.slug)

    def auth_method(self):
        return 'oauth2'

    def http_methods(self):
        return ['GET', ]

    def url(self):
        return ""


@python_2_unicode_compatible
class CustomHTTPAuthReadAPI(models.Model):

    group = models.ForeignKey(
        Group, blank=True, null=True, on_delete=models.CASCADE)
    output_format = models.CharField(max_length=4,
                                     choices=OUTPUT_CHOICES,
                                     default="json")
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    query = models.TextField(max_length=2048, default="{}",
                             verbose_name="JSON Query")
    type_mapper = models.TextField(
        max_length=2048,
        default="{}",
        verbose_name=_("Map non-string variables to numbers or Boolean"))
    sort = models.TextField(
        max_length=2048,
        default="",
        blank=True,
        verbose_name="Sort Dict",
        help_text="""e.g. [["somefield", 1], ["someotherfield", -1] ]""")

    return_keys = models.TextField(max_length=2048, default="", blank=True,
                                   help_text=_("Default is blank which "
                                               "returns all keys. Separate "
                                               "keys by white space to limit"
                                               "the keys that are returned."))

    default_limit = models.IntegerField(
        default=getattr(
            settings,
            'MONGO_LIMIT',
            200),
        help_text="Limit results to this number unless specified otherwise.",
    )
    database_name = models.CharField(max_length=100)
    collection_name = models.CharField(max_length=100)
    creation_date = models.DateField(auto_now_add=True)
    readme_md = models.TextField(max_length=4096, default="", blank=True)

    class Meta:
        get_latest_by = "creation_date"
        ordering = ('-creation_date',)

    def __str__(self):
        return "%s" % (self.slug)

    def url(self):
        return reverse('djmongo_run_custom_httpauth_read_api_by_slug', args=(self.slug,))

    def auth_method(self):
        return 'httpauth'

    def http_methods(self):
        return ['GET', ]


@python_2_unicode_compatible
class CustomIPAuthReadAPI(models.Model):

    output_format = models.CharField(max_length=4,
                                     choices=OUTPUT_CHOICES,
                                     default="json")
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    query = models.TextField(max_length=2048, default="{}",
                             verbose_name="JSON Query")
    type_mapper = models.TextField(
        max_length=2048,
        default="{}",
        verbose_name=_("Map non-string variables to numbers or Boolean"))
    sort = models.TextField(
        max_length=2048,
        default="",
        blank=True,
        verbose_name="Sort Dict",
        help_text="""e.g. [["somefield", 1], ["someotherfield", -1] ]""")

    return_keys = models.TextField(max_length=2048, default="", blank=True,
                                   help_text=_("Default is blank which "
                                               "returns all keys. Separate "
                                               "keys by white space to limit"
                                               "the keys that are returned."))
    default_limit = models.IntegerField(
        default=getattr(
            settings,
            'MONGO_LIMIT',
            200),
        help_text="Limit results to this number unless specified otherwise.",
    )
    from_ip = models.TextField(max_length=2048, default="127.0.0.1",
                               verbose_name=_("From IPs"),
                               help_text=_("Only accept requests from a IP in "
                                           "this list separated by whitespace "
                                           ". 0.0.0.0 means all."))
    database_name = models.CharField(max_length=100)
    collection_name = models.CharField(max_length=100)
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        get_latest_by = "creation_date"
        ordering = ('-creation_date',)

    def allowable_ips(self):
        allowable_ips = self.from_ip.split(" ")
        return allowable_ips

    def __str__(self):
        return "%s" % (self.slug)

    def url(self):
        return reverse('djmongo_run_custom_ipauth_read_api_by_slug', args=(self.slug,))

    def http_methods(self):
        return ['GET', ]

    def auth_method(self):
        return 'ipauth'


@python_2_unicode_compatible
class CustomPublicReadAPI(models.Model):

    output_format = models.CharField(max_length=4,
                                     choices=OUTPUT_CHOICES,
                                     default="json")
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    query = models.TextField(max_length=2048, default="{}",
                             verbose_name="JSON Query")
    type_mapper = models.TextField(
        max_length=2048,
        default="{}",
        verbose_name=_("Map non-string variables to numbers or Boolean"))
    sort = models.TextField(
        max_length=2048,
        default="",
        blank=True,
        verbose_name="Sort Dict",
        help_text="""e.g. [["somefield", 1], ["someotherfield", -1] ]""")

    return_keys = models.TextField(max_length=2048, default="", blank=True,
                                   help_text=_("Default is blank which "
                                               "returns all keys. Separate "
                                               "keys by white space to limit"
                                               "the keys that are returned."))
    default_limit = models.IntegerField(
        default=getattr(
            settings,
            'MONGO_LIMIT',
            200),
        help_text="Limit results to this number unless specified otherwise.",
    )
    database_name = models.CharField(max_length=100)
    collection_name = models.CharField(max_length=100)
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        get_latest_by = "creation_date"
        ordering = ('-creation_date',)

    def __str__(self):
        return "%s" % (self.slug)

    def url(self):
        return reverse('djmongo_run_custom_public_read_api_by_slug', args=(self.slug,))

    def http_methods(self):
        return ['GET', ]

    def auth_method(self):
        return 'public'


@python_2_unicode_compatible
class HTTPAuthReadAPI(models.Model):

    database_name = models.CharField(max_length=256)
    collection_name = models.CharField(max_length=256)
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    search_keys = models.TextField(max_length=4096, default="", blank=True,
                                   help_text="""The default, blank, returns
                                                all keys. Providing a list of
                                                keys, separated by whitespace,
                                                limits the API search to only
                                                these keys.""")
    groups = models.ManyToManyField(
        Group, blank=True, related_name="djmongo_http_auth_read_api")
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        # get_latest_by = "creation_date"
        # ordering = ('-creation_date',)
        unique_together = (('database_name', 'collection_name', 'slug'), )

    def __str__(self):
        return "%s/%s" % (self.database_name, self.collection_name)

    def url(self):
        return reverse('djmongo_api_httpauth_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'json'))

    def json_url(self):
        return reverse('djmongo_api_httpauth_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'json'))

    def csv_url(self):
        return reverse('djmongo_api_httpauth_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'csv'))

    def html_url(self):
        return reverse('djmongo_api_httpauth_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'html'))

    def http_methods(self):
        return ['GET', ]

    def auth_method(self):
        return 'httpauth'


@python_2_unicode_compatible
class PublicReadAPI(models.Model):

    database_name = models.CharField(max_length=256)
    collection_name = models.CharField(max_length=256)
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    search_keys = models.TextField(max_length=4096, default="", blank=True,
                                   help_text="""The default, blank, returns
                                                all keys. Providing a list of
                                                keys, separated by whitespace,
                                                limits the API search to only
                                                these keys.""")
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        # get_latest_by = "creation_date"
        # ordering = ('-creation_date',)
        unique_together = (('database_name', 'collection_name', 'slug'), )

    def __str__(self):
        return "%s/%s/%s" % (self.database_name, self.collection_name, self.slug)

    def url(self):
        return reverse('djmongo_api_public_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'json'))

    def json_url(self):
        return reverse('djmongo_api_public_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'json'))

    def csv_url(self):
        return reverse('djmongo_api_public_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'csv'))

    def html_url(self):
        return reverse('djmongo_api_public_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'html'))

    def http_methods(self):
        return ['GET', ]

    def auth_method(self):
        return 'public'


@python_2_unicode_compatible
class IPAuthReadAPI(models.Model):

    database_name = models.CharField(max_length=256)
    collection_name = models.CharField(max_length=256)
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    from_ip = models.TextField(max_length=2048, default="127.0.0.1",
                               verbose_name=_("From IPs"),
                               help_text=_("Only accept requests from a IP in "
                                           "this list separated by whitespace "
                                           ". 0.0.0.0 means all."))
    search_keys = models.TextField(max_length=4096, default="", blank=True,
                                   help_text="""The default, blank, returns
                                                all keys. Providing a list of
                                                keys, separated by whitespace,
                                                limits the API search to only
                                                these keys.""")
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        # get_latest_by = "creation_date"
        # ordering = ('-creation_date',)
        unique_together = (('database_name', 'collection_name', 'slug'), )

    def allowable_ips(self):
        allowable_ips = self.from_ip.split(" ")
        return allowable_ips

    def __str__(self):
        return "%s/%s/%s" % (self.database_name, self.collection_name, self.slug)

    def url(self):
        return reverse('djmongo_api_ipauth_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'json'))

    def json_url(self):
        return reverse('djmongo_api_ipauth_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'json'))

    def csv_url(self):
        return reverse('djmongo_api_ipauth_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'csv'))

    def html_url(self):
        return reverse('djmongo_api_ipauth_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'html'))

    def http_methods(self):
        return ['GET', ]

    def auth_method(self):
        return 'ipauth'


@python_2_unicode_compatible
class OAuth2ReadAPI(models.Model):

    database_name = models.CharField(max_length=256)
    collection_name = models.CharField(max_length=256)
    slug = models.SlugField(max_length=100,
                            help_text="Give your API a unique name")
    scopes = models.CharField(max_length=1024, default="*", blank=True,
                              help_text="Space delimited list of scopes required. * means no scope is required.")
    search_keys = models.TextField(max_length=4096, default="", blank=True,
                                   help_text="""The default, blank, returns
                                                all keys. Providing a list of
                                                keys, separated by whitespace,
                                                limits the API search to only
                                                these keys.""")
    readme_md = models.TextField(max_length=4096, default="", blank=True)
    creation_date = models.DateField(auto_now_add=True)

    class Meta:
        # get_latest_by = "creation_date"
        # ordering = ('-creation_date',)
        unique_together = (('database_name', 'collection_name', 'slug'), )

    def __str__(self):
        return "%s/%s/%s" % (self.database_name, self.collection_name, self.slug)

    def url(self):
        return reverse('djmongo_api_oauth2_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'json'))

    def json_url(self):
        return reverse('djmongo_api_oauth2_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'json'))

    def csv_url(self):
        return reverse('djmongo_api_oauth2_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'csv'))

    def html_url(self):
        return reverse('djmongo_api_oauth2_simple_search',
                       args=(self.database_name, self.collection_name, self.slug, 'html'))

    def http_methods(self):
        return ['GET', ]

    def auth_method(self):
        return 'oauth2'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from __future__ import absolute_import
from __future__ import unicode_literals
import sys
from django.conf import settings
from django.http import HttpResponse
from datetime import datetime
from collections import OrderedDict
import csv
import string
from ..mongoutils import get_collection_keys


def flatten_results(keylist, listresults, exclude=()):

    # create a blank list to use as our table
    rows = []
    # Make the first row
    row = OrderedDict()
    for i in keylist:
        row[i] = i
    rows.append(row)

    # write the rest of the rows.
    for i in listresults:
        # Make the other rows
        row = OrderedDict()
        for j in keylist:
            if j in i:

                # print i[j]

                if i[j]:
                    if isinstance(i[j], type([])) or \
                       isinstance(i[j], type({})) or \
                       isinstance(i[j], type(OrderedDict())):
                        row[j] = str(i[j])
                    else:
                        # Avoiding int object has no attribute encode
                        if sys.version_info[0] == 2:
                            if isinstance(i[j], (int, long, float)):
                                row[j] = str(i[j])
                            else:
                                row[j] = "".join(
                                    s for s in i[j].encode(
                                        "ascii", errors="ignore") if s in string.printable)
                        else:
                            row[j] = str(i[j])
                else:
                    row[j] = ""
            else:
                row[j] = ""
        rows.append(row)

    return rows


def tupleize(
    rows,
    alphabetize_columns=getattr(
        settings,
        'ALPHABETIZE_COLUMNS',
        False)):
    """Also alphabetizes columns and returns a tuple of tuples"""

    # define a blank list as our return object
    l = []
    for r in rows:
        row = []
        row = list(r.values())
        l.append(row)

    # alphabetize
    if alphabetize_columns:
        col = sorted(zip(*l))
        result = zip(*col)
        return result
    else:
        return l


def sort_by_columns(
    rows,
    sort_columns=getattr(
        settings,
        'SORTCOLUMNS',
        False)):

    if sort_columns:
        ncolumns = []
        columns = list(zip(*rows))
        sl = get_collection_keys()

        for s in sl:
            for c in columns:
                if c[0] == s:
                    ncolumns.append(c)

        difflist = list(set(columns) - set(ncolumns))

        for i in difflist:
            ncolumns.append(i)
        rows = zip(*ncolumns)
        lrows = []
        for r in rows:
            lrows.append(list(r))
        return lrows

    return rows


def convert_to_csv(keylist, listresults, exclude=(),
                   sort_columns=getattr(settings, 'SORTCOLUMNS', False)):
    rows = flatten_results(keylist, listresults, exclude=())
    rows = tupleize(rows)

    if sort_columns:
        # sort by our preferred column order, if specificed
        rows = sort_by_columns(rows)

    filename = datetime.now().strftime('%m-%d-%Y_%H:%M:%S') + '.csv'
    response = HttpResponse(content_type="text/csv")
    response['Content-Disposition'] = 'attachment; filename=' + filename

    writer = csv.writer(response, delimiter=',')
    for r in rows:
        # filtered_string = "".join(s for s in c if s in string.printable)
        writer.writerows([r])
    return response


def convert_to_rows(keylist, listresults, exclude=(),
                    sort_columns=getattr(settings, 'SORTCOLUMNS', False)):
    rows = flatten_results(keylist, listresults, exclude=())
    rows = tupleize(rows)

    if sort_columns:
        # sort by our preferred column order, if specificed
        rows = sort_by_columns(rows)

    return rows



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.conf import settings
import json
import sys
import csv
from datetime import datetime, date, time
from bson.code import Code
from bson.objectid import ObjectId
from bson.errors import InvalidId
from bson import json_util
from pymongo import MongoClient, DESCENDING
from collections import OrderedDict


def checkObjectId(s):
    try:
        ObjectId(s)
    except InvalidId:
        return False
    return True


def run_aggregation_pipeline(database_name, collection_name, pipeline):
    result = False
    mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                 'mongodb://localhost:27017/')
    mc = MongoClient(mongodb_client_url)
    db = mc[str(database_name)]
    collection = db[str(collection_name)]

    # explain = db.command('aggregate', collection, pipeline=pipeline, explain=True)
    # print explain
    collection.aggregate(pipeline)

    # print agg_result
    result = True
    return result


def to_json(results_dict):
    return json.dumps(results_dict, indent=4, default=json_util.default)


def normalize_results(results_dict):
    mydt = datetime.now()
    myd = date.today()
    myt = time(0, 0)

    for r in results_dict['results']:
        for k, v in r.items():
            if isinstance(r[k], type(mydt)) or \
               isinstance(r[k], type(myd)) or \
               isinstance(r[k], type(myt)):
                r[k] = v.__str__()
                # print r[k]
    return results_dict


def normalize_list(results_list):
    mydt = datetime.now()
    for r in results_list:
        for k, v in r.items():
            if isinstance(r[k], type(mydt)):
                r[k] = v.__str__()
    return results_list


def query_mongo(
        database_name,
        collection_name,
        query={},
        include_num_results="0",
        skip=0,
        sort=None,
        limit=getattr(
            settings,
            'MONGO_LIMIT',
            200),
        cast_strings_to_integers=False,
        return_keys=()):
    """return a response_dict  with a list of search results"""

    l = []
    response_dict = {}

    try:
        mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                     'mongodb://localhost:27017/')
        mc = MongoClient(mongodb_client_url, document_class=OrderedDict)

        db = mc[str(database_name)]
        collection = db[str(collection_name)]

        # Cast the query to integers
        if cast_strings_to_integers:
            query = cast_number_strings_to_integers(query)

        # print query
        if return_keys:
            return_dict = {}
            for k in return_keys:
                return_dict[k] = 1
            # print "returndict=",return_dict
            mysearchresult = collection.find(
                query, return_dict).skip(skip).limit(limit)
        else:
            mysearchresult = collection.find(query).skip(skip).limit(limit)

        if sort:
            mysearchresult.sort(sort)

        response_dict['code'] = 200
        if include_num_results == "1":
            response_dict['num_results'] = response_dict['num_results'] = int(
                mysearchresult.count(with_limit_and_skip=False))

        if include_num_results == "2":
            response_dict['num_results'] = response_dict['num_results'] = int(
                mysearchresult.count(with_limit_and_skip=True))

        response_dict['type'] = "search-results"
        for d in mysearchresult:
            d['id'] = d['_id'].__str__()
            del d['_id']
            l.append(d)
        response_dict['results'] = l

    except:
        print("Error reading from Mongo")
        print(str(sys.exc_info()))
        response_dict['num_results'] = 0
        response_dict['code'] = 500
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())

    return response_dict


def query_mongo_sort_decend(
        database_name,
        collection_name,
        query={},
        skip=0,
        limit=getattr(
            settings,
            'MONGO_LIMIT',
            200),
    return_keys=(),
        sortkey=None):
    """return a response_dict  with a list of search results in decending
    order based on a sort key
    """

    l = []
    response_dict = {}

    try:
        mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                     'mongodb://localhost:27017/')
        mc = MongoClient(mongodb_client_url, document_class=OrderedDict)

        db = mc[str(database_name)]
        collection = db[str(collection_name)]

        if return_keys:
            return_dict = {}
            for k in return_keys:
                return_dict[k] = 1
            # print "returndict=",return_dict
            mysearchresult = collection.find(
                query, return_dict).skip(skip).limit(limit).sort(
                sortkey, DESCENDING)
        else:
            mysearchresult = collection.find(query).skip(
                skip).limit(limit).sort(sortkey, DESCENDING)

        # response_dict['num_results']=int(mysearchresult.count(with_limit_and_skip=False))
        response_dict['code'] = 200
        response_dict['type'] = "search-results"
        for d in mysearchresult:
            d['id'] = d['_id'].__str__()
            del d['_id']
            l.append(d)
        response_dict['results'] = l

    except:
        print("Error reading from Mongo")
        print(str(sys.exc_info()))
        response_dict['num_results'] = 0
        response_dict['code'] = 500
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())
    return response_dict


def delete_mongo(database_name, collection_name,
                 query={}, just_one=False):
    """delete from mongo helper"""

    l = []
    response_dict = {}

    try:
        mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                     'mongodb://localhost:27017/')
        mc = MongoClient(mongodb_client_url, document_class=OrderedDict)
        db = mc[str(database_name)]
        collection = db[str(collection_name)]

        mysearchresult = collection.remove(query, just_one)

        response_dict['code'] = 200
        response_dict['type'] = "remove-confirmation"

    except:
        # print "Error reading from Mongo"
        # print str(sys.exc_info())
        response_dict['num_results'] = 0
        response_dict['code'] = 500
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())
    return response_dict


def write_mongo(document, database_name,
                collection_name, update=False):
    """Write a document to the collection. Return a response_dict containing
    the written record. Method functions as both insert or update based on update
    parameter"""

    l = []
    response_dict = {}
    try:
        mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                     'mongodb://localhost:27017/')
        mc = MongoClient(mongodb_client_url, document_class=OrderedDict)
        db = mc[str(database_name)]
        collection = db[str(collection_name)]

        # Cast the query to integers
        # if settings.CAST_ININGS_TO_INTEGERS:
        #    query = cast_number_strings_to_integers(query)

        potential_key_found = False
        existing_transaction_id = None
        existing_mongo_id = None

        # enforce non-repudiation constraint on create
        # if document.has_key("transaction_id"):
        #    existing_transaction_id = collection.find_one({'transaction_id':document['transaction_id']})
        #    if existing_transaction_id:
        #        potential_key_found = True

        if "id" in document:
            document["_id"] = ObjectId(document["id"])
            del document["id"]

        if "_id" in document:
            existing_mongo_id = collection.find_one({'_id': document['_id']})
            if existing_mongo_id:
                potential_key_found = True

        if update == False and potential_key_found == True:
            """409 conflict"""
            response_dict['code'] = 409
            response_dict['type'] = "Error"
            response_dict['results'] = []
            response_dict[
                'message'] = "Perhaps you meant to perform an update instead?"
            response_dict['errors'] = [
                "Conflict. This transaction_id has already been created.", ]
            return response_dict

        elif update and potential_key_found:  # this is an update
            # set kwargs _id to the existing_id to force to overwrite existing
            # document

            # if existing_transaction_id:
            #
            #    document['_id'] = ObjectId(existing_transaction_id['_id'])
            #    document['history']=True
            #    history_collection_name = "%s_history" % str(collection_name)
            #    history_collection   = db[str(history_collection_name)]
            #
            #    history_object = existing_transaction_id
            #    history_object['historical_id'] = existing_transaction_id['_id']
            #    del history_object['_id']
            #    #now write the record to the historical collection
            #    written_object = history_collection.insert(history_object)

            if existing_mongo_id:
                document['_id'] = ObjectId(existing_mongo_id['_id'])
                document['history'] = True
                history_collection_name = "%s_history" % str(collection_name)
                history_collection = db[str(history_collection_name)]

                # print history_collection
                # print existing_mongo_id

                history_object = existing_mongo_id

                history_object['historical_id'] = existing_mongo_id['_id']
                del history_object['_id']
                # print history_object

                # now write the record to the historical collection
                written_object = history_collection.insert(history_object)

            # update the record
            myobjectid = collection.save(document)

        else:
            # this is new so perform an insert.
            myobjectid = collection.insert(document)

        # now fetch the record we just wrote so that we write it back to the
        # DB.
        myobject = collection.find_one({'_id': myobjectid})
        response_dict['code'] = 200
        response_dict['type'] = "write-results"
        myobject['id'] = myobject['_id'].__str__()
        del myobject['_id']
        l.append(myobject)
        response_dict['results'] = l

    except:
        # print "Error reading from Mongo"
        # print str(sys.exc_info())
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())
    return response_dict


def bulk_csv_import_mongo(csvfile, database_name, collection_name,
                          delete_collection_before_import=False):
    """return a response_dict  with a list of search results"""
    """method can be insert or update"""

    l = []
    response_dict = {}
    try:
        mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                     'mongodb://localhost:27017/')
        mc = MongoClient(mongodb_client_url, document_class=OrderedDict)
        db = mconnection[database_name]
        collection = db[collection_name]

        if delete_collection_before_import:
            myobjectid = collection.remove({})

        # open the csv file.
        csvhandle = csv.reader(open(csvfile._get_path(), 'rb'), delimiter=',')

        rowindex = 0
        errors = 0
        error_list = []
        success = 0
        for row in csvhandle:

            if rowindex == 0:
                column_headers = row
                cleaned_headers = []
                for c in column_headers:
                    c = c.replace(".", "")
                    c = c.replace("$", "-")
                    c = c.replace(" ", "_")
                    cleaned_headers.append(c)
            else:

                record = OrderedDict(zip(cleaned_headers, row))
                # if there is no values, skip the key value pair
                kwargs = OrderedDict()

                # Only populate fields that are not blank.
                for k, v in record.items():
                    if v:
                        if v.isdigit():
                            kwargs[k] = int(v)
                        else:
                            kwargs[k] = v
                try:

                    myobjectid = collection.insert(kwargs)
                    success += 1
                except:
                    error_message = "Error on row " + \
                        rowindex + ". " + str(sys.exc_info())
                    error_list.append(str(sys.exc_info()))

            rowindex += 1

        if error_list:
            response_dict = {}
            response_dict['num_rows_imported'] = rowindex
            response_dict['num_rows_errors'] = len(error_list)
            response_dict['errors'] = error_list
            response_dict['code'] = 400
            response_dict['message'] = "Completed with errors"
        else:

            response_dict = {}
            response_dict['num_rows_imported'] = success
            response_dict['code'] = 200
            response_dict['message'] = "Completed."
        return response_dict

    except:
        # print "Error reading from Mongo"
        # print str(sys.exc_info())
        response_dict['num_results'] = 0
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = str(sys.exc_info())
    return response_dict


def build_non_observational_key(k):

    if str(k).__contains__("__"):
        model_field_split = str(k).split("__")
        newlabel = "%s_" % (model_field_split[0])

        field_occurence_split = str(model_field_split[1]).split("_")

        for i in field_occurence_split[:-1]:
            newlabel = "%s_%s" % (newlabel, i)
        return newlabel
    return k


def get_collection_keys(database_name, collection_name):
    l = []
    try:
        mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                     'mongodb://localhost:27017/')
        mc = MongoClient(mongodb_client_url, document_class=OrderedDict)
        db = mconnection[database_name]
        ckey_collection = "%s_keys" % (collection_name)
        collection = db[ckey_collection]
        result = collection.find({}).distinct("_id")
        for r in result:
            l.append(r)

        if getattr(settings, 'SORTCOLUMNS', False):
            nl = []  # new list list
            # sort the list according to our list

            for i in getattr(settings, 'SORTCOLUMNS', False):
                for j in l:
                    if j.__contains__(i):
                        nl.append(j)
            difflist = list(set(l) - set(nl))

            for i in difflist:
                nl.append(i)
            return nl

        else:
            return sorted(l)
    except:
        print("Error.", str(sys.exc_info()))
        return []


def build_keys_with_mapreduce(database_name, collection_name):
    map = Code("function() { "
               "    for (var key in this)"
               "        { emit(key, null); } }"
               )
    reduce = Code("function(key, stuff)"
                  "{ return null; }"
                  )

    mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                 'mongodb://localhost:27017/')
    mc = MongoClient(mongodb_client_url, document_class=OrderedDict)
    db = mc[database_name]

    collection = db[collection_name]
    result_collection_name = "%s_keys" % (collection_name)

    result = collection.map_reduce(map, reduce, result_collection_name)
    return None


def raw_query_mongo_db(kwargs, database_name, collection_name):
    # for key in kwargs:
    #    print "arg: %s: %s" % (key, kwargs[key])
    """return a result list or an empty list"""
    l = []
    response_dict = {}

    try:
        mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                     'mongodb://localhost:27017/')
        mc = MongoClient(mongodb_client_url, document_class=OrderedDict)
        db = mc[database_name]
        transactions = db[collection_name]
        mysearchresult = transactions.find(kwargs)
        mysearchcount = mysearchresult.count()
        if mysearchcount > 0:
            response_dict['code'] = 200
            for d in mysearchresult:
                l.append(d)
            response_dict['results'] = l
    except:
        # print "Error reading from Mongo"
        # print str(sys.exc_info())
        response_dict['code'] = 400

        response_dict['type'] = "Error"
        response_dict['message'] = str(sys.exc_info())
    return response_dict


def cast_number_strings_to_integers(d):
    """d is a dict"""
    for k, v in d.items():
        # print type(v)
        if determine_if_str_or_unicode(v):
            if v.isdigit():
                d[k] = int(v)
    return d


def determine_if_str_or_unicode(s):
    # if str or unicode return True, else False.
    if isinstance(s, str) or isinstance(s, unicode):
        return True
    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.shortcuts import get_object_or_404
from django.shortcuts import render
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.urls import reverse
from .forms import AggregationForm
from .models import Aggregation
from django.utils.translation import ugettext_lazy as _


def create_saved_aggregation(request, database_name=None,
                             collection_name=None):
    name = _("Create a Saved Aggregation")
    if request.method == 'POST':
        form = AggregationForm(request.POST)
        if form.is_valid():
            sa = form.save(commit=False)
            sa.user = request.user
            sa.save()

            return HttpResponseRedirect(
                reverse(
                    'djmongo_browse_saved_aggregations_w_params',
                    args=(
                        sa.database_name,
                        sa.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name},
                          )

    # This is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name,
               'form': AggregationForm(initial=idata)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context)


def run_aggregation_by_slug(request, slug):
    """Run Aggregation By Slug"""
    sa = get_object_or_404(Aggregation, slug=slug)
    sa.execute_now = True
    sa.save()
    messages.success(request, _("Saved aggregation executed."))
    return HttpResponseRedirect(
        reverse(
            'djmongo_browse_saved_aggregations_w_params',
            args=(
                sa.database_name,
                sa.collection_name)))


def delete_saved_aggregation_by_slug(request, slug):
    """Delete Saved Aggregation By Slug"""
    ss = get_object_or_404(Aggregation, slug=slug)
    ss.delete()
    messages.success(request, _("Saved aggregation deleted."))
    return HttpResponseRedirect(
        reverse(
            'djmongo_browse_saved_aggregations_w_params',
            args=(
                ss.database_name,
                ss.collection_name)))


def edit_saved_aggregation_by_slug(request, slug):
    name = _("Edit Saved Aggregation")
    ss = get_object_or_404(Aggregation, slug=slug, user=request.user)

    if request.method == 'POST':
        form = AggregationForm(request.POST, instance=ss)
        if form.is_valid():
            ss = form.save(commit=False)
            ss.user = request.user
            ss.save()
            messages.success(request, _("Aggregation edit saved."))
            return HttpResponseRedirect(
                reverse(
                    'djmongo_browse_saved_aggregations_w_params',
                    args=(
                        ss.database_name,
                        ss.collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request, 'generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })
    # this is a GET
    context = {'name': name,
               'form': AggregationForm(instance=ss)
               }
    return render(
        request,
        'djmongo/console/generic/bootstrapform.html',
        context)


def browse_saved_aggregations(request, database_name, collection_name):

    savedaggs = Aggregation.objects.filter(
        database_name=database_name,
        collection_name=collection_name)
    context = {"savedaggs": savedaggs,
               'database_name': database_name,
               'collection_name': collection_name}
    return render(
        request,
        'djmongo/console/display-saved-aggregations.html',
        context)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django import forms
from django.utils.translation import ugettext_lazy as _
from .utils import mongodb_ensure_index, create_mongo_db
import json


AUTH_TYPE_CHOICES = (('public', 'Public (NONE)'),
                     ('httpauth', 'Basic HTTP Authorization'),
                     ('ipauth', 'IP Based Authorization'),
                     ('oauth2', 'OAuth2 Authorization'))

HTTP_METHOD_CHOICES = (('write', 'Write(POST/PUT)'),
                       ('read', 'Read(GET)'),
                       ('delete', 'Delete(DELETE)'))

API_TYPE_CHOICES = (('basic', 'Basic'), ('custom', 'Custom'))


class ConfirmDropForm(forms.Form):
    name = forms.CharField(max_length=256)
    required_css_class = 'required'


class LoginForm(forms.Form):
    username = forms.CharField(max_length=30, label=_("User"))
    password = forms.CharField(widget=forms.PasswordInput, max_length=30,
                               label=_("Password"))
    required_css_class = 'required'


class EnsureIndexForm(forms.Form):
    key = forms.CharField(label=_("Key"))
    database_name = forms.CharField()
    collection_name = forms.CharField()
    required_css_class = 'required'

    def save(self, database_name, collection_name):
        key = self.cleaned_data["key"]
        result = mongodb_ensure_index(database_name, collection_name, key)
        return result


class CreateDatabaseForm(forms.Form):

    database_name = forms.CharField()
    collection_name = forms.CharField()
    initial_document = forms.CharField(
        widget=forms.Textarea,
        help_text="""You must create a document in order to
                     create a new database and collection""")
    required_css_class = 'required'

    def clean_initial_document(self):

        initial_document = self.cleaned_data.get('initial_document', '')

        try:
            djson = json.loads(initial_document)
            if not isinstance(djson, type({})):
                raise forms.ValidationError('Not a JSON object (i.e. {} )')

        except ValueError:
            raise forms.ValidationError('Invalid JSON.')

        return initial_document

    def save(self):

        result = create_mongo_db(self.cleaned_data["database_name"],
                                 self.cleaned_data["collection_name"],
                                 self.cleaned_data["initial_document"])
        return result


class DeleteForm(forms.Form):
    query = forms.CharField(widget=forms.Textarea, initial="{}")
    just_one = forms.BooleanField(required=False)
    database_name = forms.CharField()
    collection_name = forms.CharField()

    required_css_class = 'required'

    def clean_query(self):

        query = self.cleaned_data.get('query', '')

        try:
            djson = json.loads(query)
            if not isinstance(djson, type({})):
                raise forms.ValidationError("""Not a JSON
                                            object (i.e. {} )""")

        except ValueError:
            raise forms.ValidationError('Invalid JSON.')

        return query


class DocumentForm(forms.Form):
    document = forms.CharField(widget=forms.Textarea)
    database_name = forms.CharField()
    collection_name = forms.CharField()

    required_css_class = 'required'

    def clean_document(self):

        document = self.cleaned_data.get('document', '')

        try:
            djson = json.loads(document)
            if not isinstance(djson, type({})):
                raise forms.ValidationError('Not a JSON object (i.e. {} )')

        except ValueError:
            raise forms.ValidationError('Invalid JSON.')

        return document


class APIWizardForm(forms.Form):
    auth_type = forms.ChoiceField(choices=AUTH_TYPE_CHOICES,
                                  widget=forms.RadioSelect)
    http_method = forms.ChoiceField(choices=HTTP_METHOD_CHOICES,
                                    widget=forms.RadioSelect)
    api_type = forms.ChoiceField(choices=API_TYPE_CHOICES,
                                 widget=forms.RadioSelect, help_text='''A simple API that accepts get parameters and no special query (choose this option if you're unsure), Setup a read API with a custom query parameters that feed into a custom MongoDB query that you define.''')

    database_name = forms.CharField()
    collection_name = forms.CharField()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.contrib import admin
from .models import WriteAPIHTTPAuth, WriteAPIIP, WriteAPIOAuth2


class WriteAPIHTTPAuthAdmin(admin.ModelAdmin):
    list_display = ('url', 'slug', 'database_name',
                    'collection_name', 'creation_date')


admin.site.register(WriteAPIHTTPAuth, WriteAPIHTTPAuthAdmin)


class WriteAPIIPAdmin(admin.ModelAdmin):
    list_display = ('url', 'slug', 'database_name',
                    'collection_name', 'from_ip', 'creation_date')


admin.site.register(WriteAPIIP, WriteAPIIPAdmin)


class WriteAPIOAuth2Admin(admin.ModelAdmin):
    list_display = ('slug', 'database_name',
                    'collection_name', 'scopes', 'creation_date',)


admin.site.register(WriteAPIOAuth2, WriteAPIOAuth2Admin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.core.management.base import NoArgsCommand
from djmongo.aggregations.models import Aggregation


class Command(NoArgsCommand):
    help = 'Resets all aggregations scheduled_job_completed_for_today' \
           'flags to False. Set this script to run at midnight via cron.'

    def handle_noargs(self, **options):
        self.stdout.write(
            "Resetting all aggregations scheduled_job_completed_for_today "
            "flags to False.")
        aggs = Aggregation.objects.all()
        for a in aggs:
            msg = "Resetting %s" % (a.slug)
            self.stdout.write(msg)
            a.scheduled_job_completed_for_today = False
            a.save()
        self.stdout.write("Reset complete.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.db import models
from ..mongoutils import run_aggregation_pipeline
import json
from django.contrib.auth import get_user_model


class Aggregation(models.Model):

    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)

    slug = models.SlugField(max_length=100, unique=True)
    pipeline = models.TextField(max_length=20480, default="[]",
                                verbose_name="Pipeline")
    database_name = models.CharField(max_length=256)
    collection_name = models.CharField(max_length=256)
    output_collection_name = models.CharField(
        max_length=256,
        help_text="""The resulting collection.
        Do not include $out in your pipeline.""")
    creation_date = models.DateField(auto_now_add=True)

    execute_now = models.BooleanField(blank=True, default=False)

    scheduled_job_in_process = models.BooleanField(blank=True, default=False)
    scheduled_job_completed_for_today = models.BooleanField(
        blank=True, default=False)
    scheduled_job_not_ran = models.BooleanField(blank=True, default=True)

    execute_everyday = models.BooleanField(blank=True, default=False)
    execute_sunday = models.BooleanField(blank=True, default=False)
    execute_monday = models.BooleanField(blank=True, default=False)
    execute_tuesday = models.BooleanField(blank=True, default=False)
    execute_wednesday = models.BooleanField(blank=True, default=False)
    execute_thursday = models.BooleanField(blank=True, default=False)
    execute_friday = models.BooleanField(blank=True, default=False)
    execute_saturday = models.BooleanField(blank=True, default=False)
    execute_time_1 = models.TimeField(
        verbose_name="Execute Time",
        help_text="Use 24 hour time. (e.g. 20:30:00 for 8:30 pm)")
    description = models.TextField(max_length=1024, blank=True, default="")

    class Meta:
        get_latest_by = "creation_date"
        ordering = ('-creation_date',)
        verbose_name_plural = "Saved Aggregations"

    def __str__(self):
        return "%s" % (self.slug)

    def save(self, **kwargs):
        if self. execute_everyday:
            self.execute_sunday = True
            self.execute_monday = True
            self.execute_tuesday = True
            self.execute_wednesday = True
            self.execute_thursday = True
            self.execute_friday = True
            self.execute_saturday = True

        super(Aggregation, self).save(**kwargs)

        # Execute now if the flag is set to do so.
        if self.execute_now:
            # Process aggregation
            pipeline = json.loads(self.pipeline)
            output_dict = {"$out": self.output_collection_name}
            pipeline.append(output_dict)
            result = run_aggregation_pipeline(
                self.database_name, self.collection_name, pipeline)
            return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
import json
from django.shortcuts import render
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib import messages
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from .utils import (
    show_dbs,
    mongodb_drop_collection,
    mongodb_drop_database,
    mongo_delete_json_util,
    mongo_create_json_util)
from .forms import (EnsureIndexForm, DeleteForm, DocumentForm,
                    CreateDatabaseForm, ConfirmDropForm, APIWizardForm)
from bson.objectid import ObjectId
from collections import OrderedDict
from ..write.models import WriteAPIHTTPAuth, WriteAPIIP, WriteAPIOAuth2
from ..read.models import (CustomHTTPAuthReadAPI, CustomPublicReadAPI,
                           CustomOAuth2ReadAPI, CustomIPAuthReadAPI,
                           HTTPAuthReadAPI, PublicReadAPI,
                           OAuth2ReadAPI, IPAuthReadAPI)
from django.conf import settings


def api_wizard(request, database_name=None, collection_name=None):
    name = 'API Wizard Creation'

    if request.method == 'POST':
        form = APIWizardForm(request.POST, database_name, collection_name)

        if form.is_valid():
            form_data = form.cleaned_data

            # if form_data['auth_type'] == 'public' and form_data['http_method'] == 'write' and form_data['api_type'] == 'basic':
            #     return HttpResponseRedirect(reverse(''))
            # elif form_data['auth_type'] == 'public' and form_data['http_method'] == 'write' and form_data['api_type'] == 'custom':
            #     return HttpResponseRedirect(reverse(''))
            if form_data['auth_type'] == 'public' and form_data['http_method'] == 'read' and form_data['api_type'] == 'basic':
                return HttpResponseRedirect(reverse('djmongo_create_simple_api', args=('public', form_data['database_name'], form_data['collection_name'])))
            elif form_data['auth_type'] == 'public' and form_data['http_method'] == 'read' and form_data['api_type'] == 'custom':
                return HttpResponseRedirect(reverse('djmongo_create_custom_api', args=('public', form_data['database_name'], form_data['collection_name'])))
            # elif form_data['auth_type'] == 'public' and form_data['http_method'] == 'delete' and form_data['api_type'] == 'basic':
            #     return HttpResponseRedirect(reverse(''))
            # elif form_data['auth_type'] == 'public' and form_data['http_method'] == 'delete' and form_data['api_type'] == 'custom':
            #     return HttpResponseRedirect(reverse(''))
            elif form_data['auth_type'] == 'httpauth' and form_data['http_method'] == 'write' and form_data['api_type'] == 'basic':
                return HttpResponseRedirect(reverse('djmongo_create_httpauth_write_api_w_params', args=(form_data['database_name'], form_data['collection_name'])))
            elif form_data['auth_type'] == 'httpauth' and form_data['http_method'] == 'write' and form_data['api_type'] == 'custom':
                return HttpResponseRedirect(reverse('djmongo_create_httpauth_write_api_w_params', args=(form_data['database_name'], form_data['collection_name'])))

            elif form_data['auth_type'] == 'httpauth' and form_data['http_method'] == 'read' and form_data['api_type'] == 'basic':
                return HttpResponseRedirect(reverse('djmongo_create_simple_api', args=('httpauth', form_data['database_name'], form_data['collection_name'])))
            elif form_data['auth_type'] == 'httpauth' and form_data['http_method'] == 'read' and form_data['api_type'] == 'custom':
                return HttpResponseRedirect(reverse('djmongo_create_custom_api', args=('httpauth', form_data['database_name'], form_data['collection_name'])))
            # elif form_data['auth_type'] == 'httpauth' and form_data['http_method'] == 'delete' and form_data['api_type'] == 'basic':
            #     return HttpResponseRedirect(reverse(''))

            # elif form_data['auth_type'] == 'httpauth' and form_data['http_method'] == 'delete' and form_data['api_type'] == 'custom':
            #     return HttpResponseRedirect(reverse(''))
            elif form_data['auth_type'] == 'ip' and form_data['http_method'] == 'write' and form_data['api_type'] == 'basic':
                return HttpResponseRedirect(reverse('djmongo_create_ip_write_api_w_params', args=(form_data['database_name'], form_data['collection_name'])))

            elif form_data['auth_type'] == 'ip' and form_data['http_method'] == 'write' and form_data['api_type'] == 'custom':
                return HttpResponseRedirect(reverse('djmongo_create_ip_write_api_w_params', args=(form_data['database_name'], form_data['collection_name'])))
            # elif form_data['auth_type'] == 'ip' and form_data['http_method'] == 'read' and form_data['api_type'] == 'basic':
            #     return HttpResponseRedirect(reverse(''))

            # elif form_data['auth_type'] == 'ip' and form_data['http_method'] == 'read' and form_data['api_type'] == 'custom':
            #     return HttpResponseRedirect(reverse(''))
            # elif form_data['auth_type'] == 'ip' and form_data['http_method'] == 'delete' and form_data['api_type'] == 'basic':
            #     return HttpResponseRedirect(reverse(''))
            # elif form_data['auth_type'] == 'ip' and form_data['http_method'] == 'delete' and form_data['api_type'] == 'custom':
            #     return HttpResponseRedirect(reverse(''))

            elif form_data['auth_type'] == 'oauth2' and form_data['http_method'] == 'write' and form_data['api_type'] == 'basic':
                return HttpResponseRedirect(reverse('djmongo_create_oauth2_write_api_w_params', args=(form_data['database_name'], form_data['collection_name'])))

            elif form_data['auth_type'] == 'oauth2' and form_data['http_method'] == 'write' and form_data['api_type'] == 'custom':
                return HttpResponseRedirect(reverse('djmongo_create_oauth2_write_api_w_params', args=(form_data['database_name'], form_data['collection_name'])))
            # elif form_data['auth_type'] == 'oauth2' and form_data['http_method'] == 'read' and form_data['api_type'] == 'basic':
            #     return HttpResponseRedirect(reverse(''))

            # elif form_data['auth_type'] == 'oauth2' and form_data['http_method'] == 'read' and form_data['api_type'] == 'custom':
            #     return HttpResponseRedirect(reverse(''))
            # elif form_data['auth_type'] == 'oauth2' and form_data['http_method'] == 'delete' and form_data['api_type'] == 'basic':
            #     return HttpResponseRedirect(reverse(''))
            # elif form_data['auth_type'] == 'oauth2' and form_data['http_method'] == 'delete' and form_data['api_type'] == 'custom':
            #     return HttpResponseRedirect(reverse(''))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/api-list-bootstrap.html',
                          {'form': form, 'name': name})

    else:
        # this is a GET
        context = {'name': name,
                   'form': APIWizardForm(
                       initial={"collection_name": collection_name, "database_name": database_name})
                   }
        return render(request, 'djmongo/console/generic/api-list-bootstrap.html',
                      context)


def api_list(request):
    # Get all of the Read APIs
    custom_httpauth_read_apis = CustomHTTPAuthReadAPI.objects.all()
    custom_public_read_apis = CustomPublicReadAPI.objects.all()
    custom_oauth2_read_apis = CustomOAuth2ReadAPI.objects.all()
    simple_public_read_apis = PublicReadAPI.objects.all()
    simple_httpauth_read_apis = HTTPAuthReadAPI.objects.all()
    simple_auth2_read_apis = OAuth2ReadAPI.objects.all()

    # Get all of the Write APIs
    ip_write_apis = WriteAPIIP.objects.all()
    httpauth_write_apis = WriteAPIHTTPAuth.objects.all()
    oauth2_write_apis = WriteAPIOAuth2.objects.all()
    if custom_httpauth_read_apis:
        not_empty = True
    elif custom_public_read_apis:
        not_empty = True
    elif simple_public_read_apis:
        not_empty = True
    elif simple_httpauth_read_apis:
        not_empty = True
    elif ip_write_apis:
        not_empty = True
    elif httpauth_write_apis:
        not_empty = True
    elif oauth2_write_apis:
        not_empty = True
    else:
        not_empty = False

    # pdb.set_trace()
    context = {'apis': [custom_httpauth_read_apis,
                        custom_oauth2_read_apis,
                        custom_public_read_apis,
                        simple_public_read_apis,
                        simple_httpauth_read_apis,
                        httpauth_write_apis,
                        oauth2_write_apis,
                        simple_auth2_read_apis,
                        ip_write_apis],
               "not_empty": not_empty}
    return render(request, 'djmongo/console/api-list.html',
                  context)


def show_apis(request, database_name, collection_name):

    # Get all of the Read APIs
    custom_httpauth_read_apis = CustomHTTPAuthReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)
    custom_public_read_apis = CustomPublicReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)
    custom_oauth2_read_apis = CustomOAuth2ReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)
    custom_ipauth_read_apis = CustomIPAuthReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)
    simple_public_read_apis = PublicReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)
    simple_httpauth_read_apis = HTTPAuthReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)
    simple_oauth2_read_apis = OAuth2ReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)
    simple_ipauth_read_apis = IPAuthReadAPI.objects.filter(
        database_name=database_name, collection_name=collection_name)
    # Get all of the Write APIs
    ipauth_write_apis = WriteAPIIP.objects.filter(database_name=database_name,
                                                  collection_name=collection_name)
    httpauth_write_apis = WriteAPIHTTPAuth.objects.filter(
        database_name=database_name, collection_name=collection_name)

    oauth2_write_apis = WriteAPIOAuth2.objects.filter(
        database_name=database_name, collection_name=collection_name)

    context = {  # READ - GET
        "custom_httpauth_read_apis": custom_httpauth_read_apis,
        "custom_public_read_apis": custom_public_read_apis,
        "custom_ipauth_read_apis": custom_ipauth_read_apis,
        "custom_oauth2_read_apis": custom_oauth2_read_apis,
        "simple_public_read_apis": simple_public_read_apis,
        "simple_httpauth_read_apis": simple_httpauth_read_apis,
        "simple_oauth2_read_apis": simple_oauth2_read_apis,
        "simple_ipauth_read_apis": simple_ipauth_read_apis,

        # WRITE - POST/PUT
        "httpauth_write_apis": httpauth_write_apis,
        "oauth2_write_apis": oauth2_write_apis,
        "ipauth_write_apis": ipauth_write_apis,
        'database_name': database_name,
        'collection_name': collection_name}

    # TODO DELETE - DELETE/GET

    return render(request, 'djmongo/console/show-apis.html',
                  context)


def showdbs(request):

    dbs = show_dbs()
    # print(json.dumps(dbs, indent=4))
    cleaned_dbs = []
    if not dbs:
        messages.error(
            request,
            _("""Unable to connect to MongoDB.
              Check that it is running and accessible."""))

        if settings.DEBUG:
            msg = _("MONGO Connection string is: %s" % (getattr(settings, 'MONGODB_CLIENT',
                                                                'mongodb://localhost:27017/')))
            messages.error(request, msg)
    else:
        for i in dbs:
            # only keep non-system DBs.
            if i.get('name', '') != 'admin' and i.get('name', '') != 'local':
                cleaned_dbs.append(i)

    if dbs and not cleaned_dbs:
        messages.info(
            request,
            _("""You have no databases to work on. Please create one."""))

    context = {"dbs": cleaned_dbs}
    return render(request, 'djmongo/console/showdbs.html', context)


def drop_collection(request, database_name, collection_name):
    """Drop Collection"""
    name = """Retype "%s" to drop the collection""" % (collection_name)
    if request.method == 'POST':
        form = ConfirmDropForm(request.POST)
        if form.is_valid():
            name = form.cleaned_data['name']
            if name != collection_name:
                messages.error(request, _('The name did not match. \
                                         Drop operation aborted'))
                return HttpResponseRedirect(
                    reverse(
                        'djmongo_drop_collection',
                        args=(
                            database_name,
                            collection_name)))

            response = mongodb_drop_collection(database_name, collection_name)
            if response:
                errormsg = _("ERROR %s" % (response))
                messages.error(request, errormsg)
                return HttpResponseRedirect(reverse('djmongo_show_dbs'))
            else:
                messages.success(request, _("The collection was deleted."))
                return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})
    # This is a GET
    context = {'name': name,
               'form': ConfirmDropForm(
                   initial={})}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def drop_database(request, database_name):
    name = """Retype "%s" to drop the database""" % (database_name)
    if request.method == 'POST':
        form = ConfirmDropForm(request.POST)
        if form.is_valid():
            name = form.cleaned_data['name']
            if name != database_name:
                messages.error(request, _('The name did not match. \
                                         Drop operation aborted.'))
                return HttpResponseRedirect(reverse('show_dbs'))

            response = mongodb_drop_database(database_name)
            if response:
                errormsg = _("ERROR", response)
                messages.error(request, errormsg)
                return HttpResponseRedirect(reverse('djmongo_show_dbs'))
            else:
                messages.success(request, _("The database was deleted."))
                return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})
    # This is a GET
    context = {'name': name,
               'form': ConfirmDropForm(
                   initial={})}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def simple_ensure_index(request, database_name, collection_name):
    """Ensure a MongoDB index on a particular field name"""
    name = "Ensure a MongoDB index on a particular field name"

    if request.method == 'POST':
        form = EnsureIndexForm(request.POST)

        if form.is_valid():
            result = form.save(database_name, collection_name)
            messages.success(request,
                             _("Index for %s created successfully" % result))
            return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})

    else:
        # this is a GET
        context = {'name': name,
                   'form': EnsureIndexForm(
                       initial={"database_name": database_name,
                                "collection_name": collection_name})
                   }
        return render(request, 'djmongo/console/generic/bootstrapform.html',
                      context)


def create_new_database(request):
    """Create a New Mongo Database by adding a single document."""
    name = "Create a New MongoDB Database"

    if request.method == 'POST':
        form = CreateDatabaseForm(request.POST)

        if form.is_valid():
            result = form.save()
            if "error" in result:
                messages.error(
                    request, "The database creation operation failed.")
                messages.error(request, result["error"])
            else:
                messages.success(request, "Database created.")
            return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name})

    else:
        # this is a GET
        context = {'name': name,
                   'form': CreateDatabaseForm(
                       initial={"initial_document": '{ "foo" : "bar" }'
                                })
                   }
        return render(request, 'djmongo/console/generic/bootstrapform.html',
                      context)


def create_collection(request, database_name):
    """Create a new MongoDB collection by adding a single document."""
    name = "Create a New Collection"

    if request.method == 'POST':
        form = CreateDatabaseForm(request.POST)

        if form.is_valid():
            result = form.save()
            if "error" in result:
                messages.error(
                    request, _("The database creation operation failed."))
                messages.error(request, result["error"])
            else:
                messages.success(request, "Collection created.")
            return HttpResponseRedirect(reverse('djmongo_show_dbs'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})

    else:
        # this is a GET
        context = {'name': name,
                   'form': CreateDatabaseForm(
                       initial={"initial_document": '{ "foo" : "bar" }',
                                "database_name": database_name})
                   }
        return render(request, 'djmongo/console/generic/bootstrapform.html',
                      context)


def remove_data_from_collection(request, database_name, collection_name):

    name = _("""Delete select information from a MongoDB Collection
             based on a query""")

    if request.method == 'POST':
        form = DeleteForm(request.POST)
        if form.is_valid():
            query = form.cleaned_data['query']
            just_one = form.cleaned_data['just_one']

            # run the delete
            results = mongo_delete_json_util(
                query,
                database_name=database_name,
                collection_name=collection_name,
                just_one=just_one)

            # convert to json and respond.
            results_json = json.dumps(results, indent=4)
            return HttpResponse(results_json, status=int(results['code']),
                                content_type="application/json")
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name,
               'form': DeleteForm(initial=idata)}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def create_document_in_collection(request, database_name, collection_name):
    name = _("Create a Document from JSON")

    if request.method == 'POST':
        form = DocumentForm(request.POST)
        if form.is_valid():
            document = form.cleaned_data['document']
            # create the document
            results = mongo_create_json_util(document,
                                             database_name=database_name,
                                             collection_name=collection_name)

            # convert to json and respond.
            results_json = json.dumps(results, indent=4)
            return HttpResponse(results_json,
                                status=int(results['code']),
                                content_type="application/json")
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name})
    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name, 'form': DocumentForm(initial=idata)}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def update_document_in_collection(request, database_name, collection_name):
    name = _("Update a Document from JSON")

    if request.method == 'POST' or request.method == 'PUT':
        form = DocumentForm(request.POST)
        if form.is_valid():
            document = form.cleaned_data['document']
            doc = json.loads(document, object_pairs_hook=OrderedDict)
            if "_id" not in doc and "id" not in doc:
                result = {"code": 400, "type": "Error", "message": _(
                    "Updates must include either id or _id.")}
                results_json = json.dumps(result, indent=4)
                return HttpResponse(results_json, status=result['code'],
                                    content_type="application/json")

            if "_id" in doc and "id" in doc:
                result = {"code": 400, "type": "Error", "message": _(
                    "Updates cannot contain both id and _id")}
                results_json = json.dumps(result, indent=4)
                return HttpResponse(results_json, status=result['code'],
                                    content_type="application/json")

            if "id" in doc:
                doc["_id"] = ObjectId(doc["id"])
                del doc["id"]

            if "_id" in doc:
                doc["_id"] = ObjectId(doc["_id"])

            # run the update
            results = mongo_create_json_util(
                document,
                database_name=database_name,
                collection_name=collection_name,
                update=True)

            # convert to json and respond.
            results_json = json.dumps(results, indent=4)
            return HttpResponse(results_json, status=int(results['code']),
                                content_type="application/json")
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form, 'name': name})

    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}

    context = {'name': name,
               'form': DocumentForm(initial=idata)}
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)




def run_public_delete_api(request, database_name, collection_name, slug, output_type):
    pass


def run_httpauth_delete_api(request, database_name, collection_name, slug, output_type):
    pass


def run_ipauth_delete_api(request, database_name, collection_name, slug, output_type):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
import json
from django.conf import settings
from django.http import Http404
from django.shortcuts import render, get_object_or_404
from ..models import (CustomHTTPAuthReadAPI, CustomPublicReadAPI,
                      CustomIPAuthReadAPI)
from ...mongoutils import (query_mongo, to_json, normalize_results,
                           build_keys_with_mapreduce)
from ..xls_utils import convert_to_csv, convert_to_rows
from django.utils.translation import ugettext_lazy as _
from django.http import HttpResponse, HttpResponseRedirect
from datetime import datetime
from django.urls import reverse
import shlex
from django.contrib import messages


def build_keys(request, database_name, collection_name):
    """Perform the map/reduce to refresh the keys form.
       The display the custom report screen"""
    build_keys_with_mapreduce(database_name, collection_name)
    messages.success(request, _("Successfully completed MapReduce operation. "
                                "Key collection built."))
    return HttpResponseRedirect(reverse("djmongo_show_dbs"))


def prepare_search_results(request, database_name,
                           collection_name, skip=0,
                           sort=None, limit=getattr(settings,
                                                    'MONGO_LIMIT', 200),
                           return_keys=(),
                           query={}):
    # By default, do not include number of search results.
    include_num_results = "0"

    if not query:
        kwargs = {}
        for k, v in request.GET.items():
            kwargs[k] = v
            if 'limit' in kwargs:
                limit = int(kwargs['limit'])
                del kwargs['limit']
            if 'skip' in kwargs:
                skip = int(kwargs['skip'])
                del kwargs['skip']
            # Include search result numbers if asked.
            if 'include_num_results' in kwargs:
                include_num_results = kwargs['include_num_results']
                del kwargs['include_num_results']

    else:
        kwargs = query
    result = query_mongo(database_name,
                         collection_name,
                         query=kwargs,
                         include_num_results=include_num_results,
                         skip=skip, limit=limit,
                         sort=sort, return_keys=return_keys)
    return result


def simple_search(request, database_name, collection_name, slug, output_type,
                  skip=0, limit=getattr(settings, 'MONGO_LIMIT', 200),
                  sort=None, return_keys=(),
                  query={}):

    if output_type == "json":
        return search_json(request, database_name, collection_name,
                           skip=skip, limit=limit,
                           sort=sort, return_keys=return_keys,
                           query=query)

    if output_type == "html":
        return search_html(request, database_name, collection_name,
                           skip=skip, limit=limit,
                           sort=sort, return_keys=return_keys,
                           query=query)

    if output_type == "csv":
        return search_csv(request, database_name, collection_name,
                          skip=skip, limit=limit,
                          sort=sort, return_keys=return_keys,
                          query=query)

    raise Http404


def search_json(request, database_name, collection_name,
                skip=0, limit=getattr(settings, 'MONGO_LIMIT', 200),
                sort=None, return_keys=(),
                query={}):
    result = prepare_search_results(
        request,
        database_name=database_name,
        collection_name=collection_name,
        skip=skip,
        sort=sort,
        limit=limit,
        return_keys=return_keys,
        query=query)

    if int(result['code']) == 200:
        # listresults = result['results']
        jsonresults = to_json(normalize_results(result))
        return HttpResponse(jsonresults,
                            status=int(result['code']),
                            content_type="application/json")
    else:
        response = json.dumps(result, indent=4)
        return HttpResponse(response, status=int(result['code']),
                            content_type="application/json")


def search_csv(request, database_name, collection_name,
               skip=0, sort=None, limit=getattr(settings, 'MONGO_LIMIT', 200),
               return_keys=(), query={}):

    result = prepare_search_results(
        request,
        database_name=database_name,
        collection_name=collection_name,
        sort=sort,
        skip=skip,
        limit=limit,
        return_keys=return_keys,
        query=query)

    # print result.keys()

    if int(result['code']) == 200:
        listresults = result['results']
        keylist = []
        for i in listresults:
            for j in i.keys():
                if not keylist.__contains__(j):
                    keylist.append(j)

        return convert_to_csv(keylist, listresults)

    else:
        jsonresults = to_json(result)
        return HttpResponse(jsonresults, status=int(result['code']),
                            content_type="application/json")


def search_html(request, database_name, collection_name,
                sort=None, skip=0, limit=getattr(settings, 'MONGO_LIMIT', 200),
                return_keys=(),
                query={}):

    timestamp = datetime.now().strftime('%m-%d-%Y %H:%M:%S UTC')
    result = prepare_search_results(
        request,
        database_name=database_name,
        collection_name=collection_name,
        sort=sort,
        skip=skip,
        limit=limit,
        return_keys=return_keys,
        query=query)

    # print result.keys()

    if int(result['code']) == 200:
        listresults = result['results']

        keylist = []
        for i in listresults:
            for j in i.keys():
                if not keylist.__contains__(j):
                    keylist.append(j)
        context = {"rows": convert_to_rows(keylist, listresults),
                   "timestamp": timestamp}

        return render(request, 'djmongo/console/html-table.html',
                      context)

    else:
        jsonresults = to_json(result)
        return HttpResponse(jsonresults, status=int(result['code']),
                            content_type="application/json")


def run_custom_public_read_api_by_slug(
    request,
    slug,
    output_format=None,
    skip=0,
    sort=None,
    limit=getattr(
        settings,
        'MONGO_LIMIT',
        200)):
    error = False
    response_dict = {}
    ss = get_object_or_404(CustomPublicReadAPI, slug=slug)

    query = ss.query
    type_mapper = json.loads(ss.type_mapper)
    type_mapper_keys = type_mapper.keys()
    # if a GET param matches, then replace it

    for k, v in request.GET.items():
        if k in query:
            if k not in type_mapper_keys:
                quoted_value = '"%s"' % (v)
            else:
                if type_mapper[k] == "number":
                    quoted_value = '%s' % (v)
                if type_mapper[k] == "boolean":
                    if v in ('true', 'True', 'TRUE', 'T', 'Y' '1'):
                        quoted_value = 'true'
                    if v in ('false', 'False', 'F', 'FALSE', 'N', '0'):
                        quoted_value = 'false'
            query = query.replace(k, quoted_value)

    try:
        query = json.loads(query)
        if ss.sort:
                # print ss.sort
            sort = json.loads(ss.sort)

    except ValueError:
        response_dict = {}
        response_dict['num_results'] = 0
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = "Your query was not valid JSON."
        response = json.dumps(response_dict, indent=4)
        return HttpResponse(response, content_type="application/json")

    if output_format:
        if output_format not in ("json", "csv", "html"):
            response_dict[
                'message'] = "The output format must be json, csv, or html."
            error = True
        else:
            ss.output_format = output_format
    try:
        skip = int(skip)
    except ValueError:
        response_dict['message'] = "Skip must be an integer."
        error = True
    try:
        limit = int(limit)

    except ValueError:
        response_dict['message'] = "Limit must be an integer."
        error = True
    if error:
        response_dict['num_results'] = 0
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response = json.dumps(response_dict, indent=4)
        return HttpResponse(response,
                            content_type="application/json")

    # setup the list of keys for return if specified.
    key_list = ()
    if ss.return_keys:
        key_list = shlex.split(ss.return_keys)
    # print ss.query, ss.database_name, ss.collection_name

    if ss.output_format == "json":
        return search_json(
            request,
            database_name=ss.database_name,
            collection_name=ss.collection_name,
            sort=sort,
            query=query,
            skip=int(skip),
            limit=int(
                ss.default_limit),
            return_keys=key_list)

    if ss.output_format == "html":
        return search_html(
            request,
            database_name=ss.database_name,
            collection_name=ss.collection_name,
            sort=sort,
            query=query,
            skip=int(skip),
            limit=int(
                ss.default_limit),
            return_keys=key_list)

    if ss.output_format == "csv":
        return search_csv(
            request,
            database_name=ss.database_name,
            collection_name=ss.collection_name,
            query=query,
            skip=int(skip),
            limit=int(
                ss.default_limit),
            return_keys=key_list)

    # these next line "should" never execute.
    response_dict = {}
    response_dict['num_results'] = 0
    response_dict['code'] = 500
    response_dict['type'] = "Error"
    response_dict['results'] = []
    response_dict['message'] = "Something has gone wrong." + \
        response_dict['message']
    response = json.dumps(response_dict, indent=4)
    return HttpResponse(response, content_type="application/json")


def run_custom_ipauth_read_api_by_slug(
    request,
    slug,
    output_format=None,
    skip=0,
    sort=None,
    limit=getattr(
        settings,
        'MONGO_LIMIT',
        200)):
    error = False
    response_dict = {}
    ss = get_object_or_404(CustomIPAuthReadAPI, slug=slug)

    query = ss.query
    type_mapper = json.loads(ss.type_mapper)
    type_mapper_keys = type_mapper.keys()
    # if a GET param matches, then replace it

    for k, v in request.GET.items():
        if k in query:
            if k not in type_mapper_keys:
                quoted_value = '"%s"' % (v)
            else:
                if type_mapper[k] == "number":
                    quoted_value = '%s' % (v)
                if type_mapper[k] == "boolean":
                    if v in ('true', 'True', 'TRUE', 'T', 'Y' '1'):
                        quoted_value = 'true'
                    if v in ('false', 'False', 'F', 'FALSE', 'N', '0'):
                        quoted_value = 'false'
            query = query.replace(k, quoted_value)

    try:
        query = json.loads(query)
        if ss.sort:
                # print ss.sort
            sort = json.loads(ss.sort)

    except ValueError:
        response_dict = {}
        response_dict['num_results'] = 0
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = "Your query was not valid JSON."
        response = json.dumps(response_dict, indent=4)
        return HttpResponse(response, content_type="application/json")

    if output_format:
        if output_format not in ("json", "csv", "html"):
            response_dict[
                'message'] = "The output format must be json, csv, or html."
            error = True
        else:
            ss.output_format = output_format
    try:
        skip = int(skip)
    except ValueError:
        response_dict['message'] = "Skip must be an integer."
        error = True
    try:
        limit = int(limit)

    except ValueError:
        response_dict['message'] = "Limit must be an integer."
        error = True
    if error:
        response_dict['num_results'] = 0
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response = json.dumps(response_dict, indent=4)
        return HttpResponse(response,
                            content_type="application/json")

    # setup the list of keys for return if specified.
    key_list = ()
    if ss.return_keys:
        key_list = shlex.split(ss.return_keys)
    # print ss.query, ss.database_name, ss.collection_name

    if ss.output_format == "json":
        return search_json(
            request,
            database_name=ss.database_name,
            collection_name=ss.collection_name,
            sort=sort,
            query=query,
            skip=int(skip),
            limit=int(
                ss.default_limit),
            return_keys=key_list)

    if ss.output_format == "html":
        return search_html(
            request,
            database_name=ss.database_name,
            collection_name=ss.collection_name,
            sort=sort,
            query=query,
            skip=int(skip),
            limit=int(
                ss.default_limit),
            return_keys=key_list)

    if ss.output_format == "csv":
        return search_csv(
            request,
            database_name=ss.database_name,
            collection_name=ss.collection_name,
            query=query,
            skip=int(skip),
            limit=int(
                ss.default_limit),
            return_keys=key_list)

    # these next line "should" never execute.
    response_dict = {}
    response_dict['num_results'] = 0
    response_dict['code'] = 500
    response_dict['type'] = "Error"
    response_dict['results'] = []
    response_dict['message'] = "Something has gone wrong." + \
        response_dict['message']
    response = json.dumps(response_dict, indent=4)
    return HttpResponse(response, content_type="application/json")


def run_custom_httpauth_read_api_by_slug(
    request,
    slug,
    output_format=None,
    skip=0,
    sort=None,
    limit=getattr(
        settings,
        'MONGO_LIMIT',
        200)):
    error = False
    response_dict = {}
    ss = get_object_or_404(CustomHTTPAuthReadAPI, slug=slug)

    if not request.user.is_authenticated():
        response_dict = {}
        response_dict['num_results'] = 0
        response_dict['code'] = 401
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = "Not Found. Perhaps you need to log in?"
        response = json.dumps(response_dict, indent=4)
        return HttpResponse(response, content_type="application/json")
    # If a group is defined check that user is in the a group.
    elif ss.group:
        if not request.user.groups.filter(name__in=[ss.group, ]).exists():
            response_dict = {}
            response_dict['num_results'] = 0
            response_dict['code'] = 401
            response_dict['type'] = "Error"
            response_dict['results'] = []
            response_dict[
                'message'] = "You do not have permission to run this search."
            response = json.dumps(response_dict, indent=4)
            return HttpResponse(response, content_type="application/json")

    # All okay, so run the query
    query = ss.query
    type_mapper = json.loads(ss.type_mapper)
    type_mapper_keys = type_mapper.keys()
    # if a GET param matches, then replace it

    for k, v in request.GET.items():
        if k in query:
            if k not in type_mapper_keys:
                quoted_value = '"%s"' % (v)
            else:
                if type_mapper[k] == "number":
                    quoted_value = '%s' % (v)
                if type_mapper[k] == "boolean":
                    if v in ('true', 'True', 'TRUE', 'T', 'Y' '1'):
                        quoted_value = 'true'
                    if v in ('false', 'False', 'F', 'FALSE', 'N', '0'):
                        quoted_value = 'false'
            query = query.replace(k, quoted_value)

    try:
        query = json.loads(query)
        if ss.sort:
                # print ss.sort
            sort = json.loads(ss.sort)

    except ValueError:
        response_dict = {}
        response_dict['num_results'] = 0
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response_dict['message'] = "Your query was not valid JSON."
        response = json.dumps(response_dict, indent=4)
        return HttpResponse(response, content_type="application/json")

    if output_format:
        if output_format not in ("json", "csv", "html"):
            response_dict[
                'message'] = "The output format must be json, csv, or html."
            error = True
        else:
            ss.output_format = output_format
    try:
        skip = int(skip)
    except ValueError:
        response_dict['message'] = "Skip must be an integer."
        error = True
    try:
        limit = int(limit)

    except ValueError:
        response_dict['message'] = "Limit must be an integer."
        error = True
    if error:
        response_dict['num_results'] = 0
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['results'] = []
        response = json.dumps(response_dict, indent=4)
        return HttpResponse(response,
                            content_type="application/json")

    # setup the list of keys for return if specified.
    key_list = ()
    if ss.return_keys:
        key_list = shlex.split(ss.return_keys)
    # print ss.query, ss.database_name, ss.collection_name

    if ss.output_format == "json":
        return search_json(
            request,
            database_name=ss.database_name,
            collection_name=ss.collection_name,
            sort=sort,
            query=query,
            skip=int(skip),
            limit=int(
                ss.default_limit),
            return_keys=key_list)

    if ss.output_format == "html":
        return search_html(
            request,
            database_name=ss.database_name,
            collection_name=ss.collection_name,
            sort=sort,
            query=query,
            skip=int(skip),
            limit=int(
                ss.default_limit),
            return_keys=key_list)

    if ss.output_format == "csv":
        return search_csv(
            request,
            database_name=ss.database_name,
            collection_name=ss.collection_name,
            query=query,
            skip=int(skip),
            limit=int(
                ss.default_limit),
            return_keys=key_list)

    # these next line "should" never execute.
    response_dict = {}
    response_dict['num_results'] = 0
    response_dict['code'] = 500
    response_dict['type'] = "Error"
    response_dict['results'] = []
    response_dict['message'] = "Something has gone wrong." + \
        response_dict['message']
    response = json.dumps(response_dict, indent=4)
    return HttpResponse(response, content_type="application/json")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

import json
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib.auth import authenticate, login, logout
from django.urls import reverse
from django.shortcuts import render
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from .forms import (LoginForm, UserCreationForm, UserUpdateForm,
                    APIUserUpdateForm, APIUserCreationForm)
from .decorators import json_login_required, access_required
from .models import SocialGraph
from django.utils.translation import ugettext_lazy as _
from django.contrib import messages
from django.contrib.auth.forms import SetPasswordForm
from django.contrib.auth.models import User


def simple_logout(request):
    logout(request)
    messages.success(request, _("Logged out successfully."))
    return HttpResponseRedirect(reverse('djmongo_login'))


def simple_login(request):
    if request.method == 'POST':
        form = LoginForm(request.POST)
        if form.is_valid():
            username = form.cleaned_data['username']
            password = form.cleaned_data['password']
            user = authenticate(username=username, password=password)

            if user is not None:
                if user.is_active:
                    login(request, user)

                    next = request.GET.get('next', '')
                    if next:
                        # If a next is in the URL, then go there
                        return HttpResponseRedirect(next)
                    # otherwise just go to home.
                    return HttpResponseRedirect(reverse('djmongo_show_dbs'))
                else:
                    # The user exists but is_active=False
                    messages.error(
                        request, _("""Your account is inactive
                                   so you may not log in."""))

                    return render(request, 'djmongo/console/login2.html',
                                  {'form': form})
            else:
                messages.error(request, _("Invalid username or password."))

                return render(request, 'djmongo/console/login2.html',
                              {'form': form})

        else:
            return render(request,
                          'djmongo/console/login2.html',
                          {'form': form})
    # this is a GET
    return render(request, 'djmongo/console/login2.html',
                  {'form': LoginForm()})


@json_login_required
def api_test_credentials(request):
    message = _("Your API credentials for user %s are valid.") \
        % (request.user)
    jsonstr = {"code": 200, "message": message}
    jsonstr = json.dumps(jsonstr, indent=4,)
    return HttpResponse(jsonstr, status=200, content_type="application/json")


@json_login_required
@access_required("create-other-users")
def api_delete_user(request, email):
    try:
        u = User.objects.get(email=email)
    except User.DoesNotExist:
        message = "User %s does not exist." % (email)
        jsond = {"code": 404, "message": message}
        jsonstr = json.dumps(jsond, indent=4,)
        return HttpResponse(
            jsonstr,
            status=404,
            content_type="application/json")

    u.delete()
    message = "User %s deleted." % (email)
    jsond = {"code": 200, "message": message}
    jsonstr = json.dumps(jsond, indent=4,)
    return HttpResponse(jsonstr, status=200, content_type="application/json")


@login_required
def user_create(request):
    print(request.user)

    name = _("Create User Account")
    if request.method == 'POST':
        form = UserCreationForm(request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('home'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request, 'generic/bootstrapform.html',
                          {'form': form, 'name': name})
    # this is a GET
    context = {'name': name,
               'form': UserCreationForm()
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


@login_required
@access_required("create-other-users")
def user_update(request):
    name = _("Update User Account")
    if request.method == 'POST':
        form = UserUpdateForm(request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('home'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })
    # this is a GET
    context = {'name': name,
               'form': UserUpdateForm(instance=request.user)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


@json_login_required
@access_required("create-other-users")
def api_read_user(request, email):

    try:
        u = User.objects.get(email=email)
    except User.DoesNotExist:
        message = _("User %s does not exist.") % (email)
        jsond = {"code": 404, "message": message}
        jsonstr = json.dumps(jsond, indent=4,)
        return HttpResponse(
            jsonstr,
            status=404,
            content_type="application/json")

    user = {"first_name": u.first_name, "last_name": u.last_name,
            "username": u.username, "email": email,
            "date_joined": str(u.date_joined)}
    jsond = {"code": 200, "user": user}
    jsonstr = json.dumps(jsond, indent=4,)
    return HttpResponse(jsonstr, status=200,
                        content_type="application/json")


@login_required
@access_required("create-other-users")
def user_password(request):
    name = _("Update Password")
    if request.method == 'POST':
        form = SetPasswordForm(request.user, request.POST)

        if form.is_valid():
            form.save()
            messages.success(request, _("Password successfully updated."))
            return HttpResponseRedirect(reverse('home'))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })

    # this is a GET
    context = {'name': name,
               'form': SetPasswordForm(user=request.user)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


@json_login_required
@csrf_exempt
@access_required("create-other-users")
def api_user_create(request):
    name = _("API Create User Account")
    if request.method == 'POST':
        form = APIUserCreationForm(request.POST)
        if form.is_valid():
            result = form.save()
            # our new user was created so lets go ahead and
            # create a social graph between the creator and the new user
            grantor = User.objects.get(username=result['user']['username'])
            SocialGraph.objects.create(grantor=grantor, grantee=request.user)
            SocialGraph.objects.create(grantor=grantor, grantee=grantor)
            jsonstr = result
            jsonstr = json.dumps(jsonstr, indent=4,)
            return HttpResponse(
                jsonstr,
                status=200,
                content_type="application/json")
        else:
            # the form had errors
            errors = []
            for nferror in form.non_field_errors():
                errors.append({'non_field': nferror})
            for k, v in form._errors.items():
                error = {'field': k, 'description': v}
                errors.append(error)
            jsonstr = {"code": 400,
                       "message": _("User creation failed due to errors."),
                       "errors": errors}
            jsonstr = json.dumps(jsonstr, indent=4,)
            return HttpResponse(
                jsonstr,
                status=400,
                content_type="application/json")
    # this is an HTTP GET
    return render(request, 'djmongo/accounts/create.html',
                  {'name': name, 'form': APIUserCreationForm()})


@json_login_required
@csrf_exempt
@access_required("create-other-users")
def api_user_update(request):
    name = _("API Update User Account")
    if request.method == 'POST':

        if 'email' not in request.POST:
            jsonstr = {
                "code": 400,
                "message": _("Update did not identify the user by email."),
                "errors": [
                    _("Update did not identify the user by email."),
                ]}
            jsonstr = json.dumps(jsonstr, indent=4,)
            return HttpResponse(
                jsonstr,
                status=400,
                content_type="application/json")

        try:
            user = User.objects.get(email=request.POST['email'])
        except:

            msg = _("User %s not found. Did you mean to perform a create?")\
                % (request.POST['email'])
            jsonstr = {"code": 404,
                       "message": "User not found.",
                       "errors": [msg, ]}
            jsonstr = json.dumps(jsonstr, indent=4,)
            return HttpResponse(
                jsonstr,
                status=400,
                content_type="application/json")

        form = APIUserUpdateForm(request.POST, instance=user)

        if form.is_valid():
            result = form.save()
            jsonstr = result
            jsonstr = json.dumps(jsonstr, indent=4,)
            return HttpResponse(
                jsonstr,
                status=200,
                content_type="application/json")
        else:
            # the form had errors # CHECK
            errors = []
            if form.non_field_errors():
                for nferror in form.non_field_errors():
                    errors.append({'non_field': nferror})

            for k, v in form._errors.items():
                error = {'field': k, 'description': v}
                errors.append(error)
            jsonstr = {"code": 400,
                       "message": _("User update failed due to errors."),
                       "errors": errors}
            jsonstr = json.dumps(jsonstr, indent=4,)
            return HttpResponse(
                jsonstr,
                status=400,
                content_type="application/json")
    # this is an HTTP GET
    return render(request, 'djmongo/accounts/create.html',
                  {'name': name, 'form': APIUserUpdateForm()})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.contrib import admin
from .models import (PublicReadAPI, HTTPAuthReadAPI,
                     OAuth2ReadAPI, IPAuthReadAPI,
                     CustomHTTPAuthReadAPI, CustomPublicReadAPI,
                     CustomIPAuthReadAPI, CustomOAuth2ReadAPI)


class CustomOAuth2ReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name',
                    'url')


admin.site.register(CustomOAuth2ReadAPI, CustomOAuth2ReadAPIAdmin)


class CustomHTTPAuthReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name',
                    'url')


admin.site.register(CustomHTTPAuthReadAPI, CustomHTTPAuthReadAPIAdmin)


class CustomIPAuthReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name',
                    'url')


admin.site.register(CustomIPAuthReadAPI, CustomIPAuthReadAPIAdmin)


class CustomPublicReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name',
                    'url')


admin.site.register(CustomPublicReadAPI, CustomPublicReadAPIAdmin)


class HTTPAuthReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name', 'json_url')


admin.site.register(HTTPAuthReadAPI, HTTPAuthReadAPIAdmin)


class IPAuthReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name', 'json_url')


admin.site.register(IPAuthReadAPI, IPAuthReadAPIAdmin)


class PublicReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name', 'json_url',)


admin.site.register(PublicReadAPI, PublicReadAPIAdmin)


class OAuth2ReadAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name', 'json_url',)


admin.site.register(OAuth2ReadAPI, OAuth2ReadAPIAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.contrib import admin
from .models import Aggregation


class AggregationAdmin(admin.ModelAdmin):
    list_display = (
        'slug',
        'user',
        'database_name',
        'collection_name',
        'output_collection_name',
        'creation_date')


admin.site.register(Aggregation, AggregationAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.conf.urls import url
from .urls import urlpatterns
from apps.dot_ext.decorators import capability_protected_resource
from oauth2_provider.decorators import protected_resource
from .views.run import simple_search, run_custom_public_read_api_by_slug

urlpatterns += [

    url(r'^api/oauth2/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)/(?P<slug>[^.]+).(?P<output_type>[^/]+)$',
        capability_protected_resource()(simple_search), name="djmongo_api_oauth2_simple_search"),

    url(r'^api/custom/oauth2/(?P<slug>\S+)$',
        capability_protected_resource()(run_custom_public_read_api_by_slug),
        name="djmongo_run_custom_oauth2_read_api_by_slug"),

]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.contrib.auth.decorators import login_required
from django.conf.urls import url
from .views import write_to_collection_httpauth, write_to_collection_ip_auth
from .views import (create_oauth2_write_api, edit_oauth2_write_api,
                    delete_oauth2_write_api)
from .views import browse_ip_write_apis, browse_httpauth_write_apis
from .views import create_httpauth_write_api, create_ip_write_api
from .views import delete_httpauth_write_api, delete_ip_write_api
from .views import edit_httpauth_write_api, edit_ip_write_api

urlpatterns = [

    # Call / Run the write API
    url(r'^api/httpauth/(?P<slug>[^/]+)$',
        write_to_collection_httpauth,
        name="djmongo_api_write_to_collection_with_httpauth"),

    url(r'^api/ip/(?P<slug>[^/]+)$',
        write_to_collection_ip_auth,
        name="djmongo_api_write_to_collection_with_ip"),

    # Browse existing APIS in the UI
    url(r'^ip/browse-write-apis/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(browse_ip_write_apis),
        name="djmongo_browse_ip_write_apis_w_params"),

    url(r'^httpauth/browse-write-apis/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(browse_httpauth_write_apis),
        name="djmongo_browse_httpauth_write_apis_w_params"),

    url(r'^ip/browse-write-apis$',
        login_required(browse_ip_write_apis),
        name="djmongo_browse_ip_write_apis"),

    url(r'^httpauth/browse-write-apis$',
        login_required(browse_httpauth_write_apis),
        name="djmongo_browse_httpauth_write_apis"),
    # Create new APIs --------------------------------------------
    url(r'^httpauth/create-write-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(create_httpauth_write_api),
        name="djmongo_create_httpauth_write_api_w_params"),

    url(r'^oauth2/create-write-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(create_oauth2_write_api),
        name="djmongo_create_oauth2_write_api_w_params"),

    url(r'^ip/create-write-api$',
        login_required(create_ip_write_api),
        name="djmongo_create_ip_write_api"),

    url(r'^httpauth/create-write-api$',
        login_required(create_httpauth_write_api),
        name="djmongo_create_httpauth_write_api"),

    url(r'^ip/create-write-api/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(create_ip_write_api),
        name="djmongo_create_ip_write_api_w_params"),

    # Edit existing write APIs ---------------------
    url(r'^ip/edit-write-api/(?P<slug>[^/]+)$',
        login_required(edit_ip_write_api),
        name="djmongo_edit_ip_write_api"),

    url(r'^httpauth/edit-write-api/(?P<slug>[^/]+)$',
        login_required(edit_httpauth_write_api),
        name="djmongo_edit_httpauth_write_api"),

    url(r'^oauth2/edit-write-api/(?P<slug>[^/]+)$',
        login_required(edit_oauth2_write_api),
        name="djmongo_edit_oauth2_write_api"),

    # Delete existing write APIs ----------------------
    url(r'^ip/delete-write-api/(?P<slug>[^/]+)$',
        login_required(delete_ip_write_api),
        name="djmongo_delete_ip_write_api"),

    url(r'^httpauth/delete-write-api/(?P<slug>[^/]+)$',
        login_required(delete_httpauth_write_api),
        name="djmongo_delete_httpauth_write_api"),


    url(r'^oauth2/delete-write-api/(?P<slug>[^/]+)$',
        login_required(delete_oauth2_write_api),
        name="djmongo_delete_oauth2_write_api"),


]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.conf import settings
from django.http import Http404
from django.shortcuts import render, get_object_or_404
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from ..models import HTTPAuthDeleteAPI, PublicDeleteAPI, OAuth2DeleteAPI, IPAuthDeleteAPI
from collections import OrderedDict
from ..forms import HTTPAuthDeleteAPIForm, IPAuthDeleteAPIForm, OAuth2DeleteAPIForm, PublicDeleteAPIForm

model_map = OrderedDict()

model_map["public"] = {"model": PublicDeleteAPI,
                       "name": "Public Delete API",
                       "form": PublicDeleteAPIForm}

model_map["httpauth"] = {"model": HTTPAuthDeleteAPI,
                         "name": "HTTP Auth Delete API",
                         "form": HTTPAuthDeleteAPIForm}

model_map["ipauth"] = {"model": IPAuthDeleteAPI,
                       "name": "IP Auth Delete API",
                       "form": HTTPAuthDeleteAPIForm}

model_map["oauth2"] = {"model": OAuth2DeleteAPI,
                       "name": "OAuth2 Delete API",
                       "form": OAuth2DeleteAPIForm}


def create_delete_api(request, auth_type, database_name, collection_name):

    if auth_type not in model_map.keys():
        raise Http404

    mymodel = model_map[auth_type]['model']
    myform = model_map[auth_type]['form']
    name = "Create a %s" % (model_map[auth_type]['name'])

    if request.method == 'POST':
        form = myform(request.POST)
        if form.is_valid():
            form.save()
            messages.success(
                request, _("The %s was created." % (name)))

            return HttpResponseRedirect(
                reverse('djmongo_show_apis',
                        args=(database_name,
                              collection_name)))
        else:
            # The form is invalid
            messages.error(
                request, _("Please correct the errors in the form."))
            return render(request,
                          'djmongo/console/generic/bootstrapform.html',
                          {'form': form,
                           'name': name,
                           })
    # this is a GET
    idata = {'database_name': database_name,
             'collection_name': collection_name}
    context = {'name': name,
               'form': form(initial=idata)
               }
    return render(request, 'djmongo/console/generic/bootstrapform.html',
                  context)


def edit_delete_api(request, auth_type, database_name, collection_name, slug):
    pass


def delete_delete_api(request, auth_type, database_name, collection_name, slug):
    pass



# Generated by Django 2.2.4 on 2019-11-05 17:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='CustomIPAuthReadAPI',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('output_format', models.CharField(choices=[('json', 'JSON'), ('html', 'HTML'), ('csv', 'Comma Separated Value (.csv)')], default='json', max_length=4)),
                ('slug', models.SlugField(help_text='Give your API a unique name', max_length=100)),
                ('query', models.TextField(default='{}', max_length=2048, verbose_name='JSON Query')),
                ('type_mapper', models.TextField(default='{}', max_length=2048, verbose_name='Map non-string variables to numbers or Boolean')),
                ('sort', models.TextField(blank=True, default='', help_text='e.g. [["somefield", 1], ["someotherfield", -1] ]', max_length=2048, verbose_name='Sort Dict')),
                ('return_keys', models.TextField(blank=True, default='', help_text='Default is blank which returns all keys. Separate keys by white space to limitthe keys that are returned.', max_length=2048)),
                ('default_limit', models.IntegerField(default=200, help_text='Limit results to this number unless specified otherwise.')),
                ('from_ip', models.TextField(default='127.0.0.1', help_text='Only accept requests from a IP in this list separated by whitespace . 0.0.0.0 means all.', max_length=2048, verbose_name='From IPs')),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
                ('creation_date', models.DateField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
            },
        ),
        migrations.CreateModel(
            name='CustomPublicReadAPI',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('output_format', models.CharField(choices=[('json', 'JSON'), ('html', 'HTML'), ('csv', 'Comma Separated Value (.csv)')], default='json', max_length=4)),
                ('slug', models.SlugField(help_text='Give your API a unique name', max_length=100)),
                ('query', models.TextField(default='{}', max_length=2048, verbose_name='JSON Query')),
                ('type_mapper', models.TextField(default='{}', max_length=2048, verbose_name='Map non-string variables to numbers or Boolean')),
                ('sort', models.TextField(blank=True, default='', help_text='e.g. [["somefield", 1], ["someotherfield", -1] ]', max_length=2048, verbose_name='Sort Dict')),
                ('return_keys', models.TextField(blank=True, default='', help_text='Default is blank which returns all keys. Separate keys by white space to limitthe keys that are returned.', max_length=2048)),
                ('default_limit', models.IntegerField(default=200, help_text='Limit results to this number unless specified otherwise.')),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
                ('creation_date', models.DateField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
            },
        ),
        migrations.CreateModel(
            name='PublicReadAPI',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('database_name', models.CharField(max_length=256)),
                ('collection_name', models.CharField(max_length=256)),
                ('slug', models.SlugField(help_text='Give your API a unique name', max_length=100)),
                ('search_keys', models.TextField(blank=True, default='', help_text='The default, blank, returns\n                                                all keys. Providing a list of\n                                                keys, separated by whitespace,\n                                                limits the API search to only\n                                                these keys.', max_length=4096)),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
                ('creation_date', models.DateField(auto_now_add=True)),
            ],
            options={
                'unique_together': {('database_name', 'collection_name', 'slug')},
            },
        ),
        migrations.CreateModel(
            name='OAuth2ReadAPI',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('database_name', models.CharField(max_length=256)),
                ('collection_name', models.CharField(max_length=256)),
                ('slug', models.SlugField(help_text='Give your API a unique name', max_length=100)),
                ('scopes', models.CharField(blank=True, default='*', help_text='Space delimited list of scopes required. * means no scope is required.', max_length=1024)),
                ('search_keys', models.TextField(blank=True, default='', help_text='The default, blank, returns\n                                                all keys. Providing a list of\n                                                keys, separated by whitespace,\n                                                limits the API search to only\n                                                these keys.', max_length=4096)),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
                ('creation_date', models.DateField(auto_now_add=True)),
            ],
            options={
                'unique_together': {('database_name', 'collection_name', 'slug')},
            },
        ),
        migrations.CreateModel(
            name='IPAuthReadAPI',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('database_name', models.CharField(max_length=256)),
                ('collection_name', models.CharField(max_length=256)),
                ('slug', models.SlugField(help_text='Give your API a unique name', max_length=100)),
                ('from_ip', models.TextField(default='127.0.0.1', help_text='Only accept requests from a IP in this list separated by whitespace . 0.0.0.0 means all.', max_length=2048, verbose_name='From IPs')),
                ('search_keys', models.TextField(blank=True, default='', help_text='The default, blank, returns\n                                                all keys. Providing a list of\n                                                keys, separated by whitespace,\n                                                limits the API search to only\n                                                these keys.', max_length=4096)),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
                ('creation_date', models.DateField(auto_now_add=True)),
            ],
            options={
                'unique_together': {('database_name', 'collection_name', 'slug')},
            },
        ),
        migrations.CreateModel(
            name='CustomOAuth2ReadAPI',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('output_format', models.CharField(choices=[('json', 'JSON'), ('html', 'HTML'), ('csv', 'Comma Separated Value (.csv)')], default='json', max_length=4)),
                ('slug', models.SlugField(help_text='Give your API a unique name', max_length=100)),
                ('scopes', models.CharField(blank=True, default='*', help_text='Space delimited list of scopes required. * means no scope is required.', max_length=1024)),
                ('query', models.TextField(default='{}', max_length=2048, verbose_name='JSON Query')),
                ('type_mapper', models.TextField(default='{}', max_length=2048, verbose_name='Map non-string variables to numbers or Boolean')),
                ('sort', models.TextField(blank=True, default='', help_text='e.g. [["somefield", 1], ["someotherfield", -1] ]', max_length=2048, verbose_name='Sort Dict')),
                ('return_keys', models.TextField(blank=True, default='', help_text='Default is blank which returns all keys. Separate keys by white space to limitthe keys that are returned.', max_length=2048)),
                ('default_limit', models.IntegerField(default=200, help_text='Limit results to this number unless specified otherwise.')),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
            ],
            options={
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
                'unique_together': {('database_name', 'collection_name', 'slug')},
            },
        ),
        migrations.CreateModel(
            name='CustomHTTPAuthReadAPI',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('output_format', models.CharField(choices=[('json', 'JSON'), ('html', 'HTML'), ('csv', 'Comma Separated Value (.csv)')], default='json', max_length=4)),
                ('slug', models.SlugField(help_text='Give your API a unique name', max_length=100)),
                ('query', models.TextField(default='{}', max_length=2048, verbose_name='JSON Query')),
                ('type_mapper', models.TextField(default='{}', max_length=2048, verbose_name='Map non-string variables to numbers or Boolean')),
                ('sort', models.TextField(blank=True, default='', help_text='e.g. [["somefield", 1], ["someotherfield", -1] ]', max_length=2048, verbose_name='Sort Dict')),
                ('return_keys', models.TextField(blank=True, default='', help_text='Default is blank which returns all keys. Separate keys by white space to limitthe keys that are returned.', max_length=2048)),
                ('default_limit', models.IntegerField(default=200, help_text='Limit results to this number unless specified otherwise.')),
                ('database_name', models.CharField(max_length=100)),
                ('collection_name', models.CharField(max_length=100)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
                ('group', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.Group')),
            ],
            options={
                'ordering': ('-creation_date',),
                'get_latest_by': 'creation_date',
            },
        ),
        migrations.CreateModel(
            name='HTTPAuthReadAPI',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('database_name', models.CharField(max_length=256)),
                ('collection_name', models.CharField(max_length=256)),
                ('slug', models.SlugField(help_text='Give your API a unique name', max_length=100)),
                ('search_keys', models.TextField(blank=True, default='', help_text='The default, blank, returns\n                                                all keys. Providing a list of\n                                                keys, separated by whitespace,\n                                                limits the API search to only\n                                                these keys.', max_length=4096)),
                ('readme_md', models.TextField(blank=True, default='', max_length=4096)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('groups', models.ManyToManyField(blank=True, related_name='djmongo_http_auth_read_api', to='auth.Group')),
            ],
            options={
                'unique_together': {('database_name', 'collection_name', 'slug')},
            },
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.conf import settings
import json
import sys
from pymongo import MongoClient
from collections import OrderedDict
from ..mongoutils import delete_mongo, write_mongo
import pymongo


def client_connector(mongodb_client=getattr(settings, 'MONGODB_CLIENT',
                                            'mongodb://localhost:27017/')):
    client = MongoClient(
        mongodb_client,
        connectTimeoutMS=2000,
        serverSelectionTimeoutMS=2000)
    try:
        client.admin.command(
            'isMaster',
            connectTimeoutMS=2000,
            serverSelectionTimeoutMS=2000)
    except pymongo.errors.ConnectionFailure:
        client = None
    return client


def show_dbs():
    """return a list of all dbs and related collections.
    Return an empty list on error.
    """

    collection_list = []
    mc = client_connector()
    if not mc:
        # The client couldn't connect
        return ()

    dbs = mc.database_names()
    for d in dbs:
        dbc = mc[d]
        collections = dbc.collection_names()
        collections = remove_values_from_list(collections, "system.indexes")
        collection_list.append({"name": d, "collections": collections})
    return tuple(collection_list)


def mongo_delete_json_util(database_name, collection_name, query={},
                           just_one=False):

    response_dict = {}
    valid_json = True
    try:
        query = json.loads(query, object_pairs_hook=OrderedDict)
        if not isinstance(query, type(OrderedDict())):
            valid_json = False
    except ValueError:
        valid_json = False

    if not valid_json:
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict['message'] = "Your query was not valid" \
                                   " JSON or not a dictionary (i.e.{})."
    else:
        # valid json so run the delete
        response_dict = delete_mongo(
            query,
            database_name=database_name,
            collection_name=collection_name,
            just_one=False)

    return response_dict


def mongo_create_json_util(document, database_name,
                           collection_name):

    response_dict = {}
    valid_json = True

    try:
        document = json.loads(document, object_pairs_hook=OrderedDict)
        if not isinstance(document, type(OrderedDict())):
            valid_json = False
    except ValueError:
        valid_json = False

    if not valid_json:
        response_dict['code'] = 400
        response_dict['type'] = "Error"
        response_dict[
            'message'] = "Your query was not valid JSON" \
                         " or not a JSON object (i.e.{})."
    else:
        # valid json so run the delete
        response_dict = write_mongo(document,
                                    database_name=database_name,
                                    collection_name=collection_name)
    return response_dict


def remove_values_from_list(the_list, val):
    return [value for value in the_list if value != val]


def create_mongo_db(database_name, collection_name, initial_document):
    response_dict = {}
    try:
        mongodb_client_url = getattr(settings, 'MONGODB_CLIENT',
                                     'mongodb://localhost:27017/')
        mc = MongoClient(mongodb_client_url, document_class=OrderedDict)
        db = mc[str(database_name)]
        collection = db[str(collection_name)]
        d = json.loads(initial_document, object_pairs_hook=OrderedDict)
        collection.save(d)

    except:
        # error connecting to mongodb
        response_dict['error'] = str(sys.exc_info())

    return response_dict


def mongodb_ensure_index(database_name, collection_name, keys):
    """ ensure index """
    try:
        mc = MongoClient(getattr(settings, 'MONGODB_CLIENT',
                                 'mongodb://localhost:27017/'))
        dbs = mc[database_name]
        dbc = dbs[collection_name]

        dbc.ensure_index(keys)
        # print "success"
        return ""
    except:
        # error connecting to mongodb
        # print str(sys.exc_info())
        return str(sys.exc_info())


def mongodb_drop_collection(database_name, collection_name):
    """Drop Collection"""
    try:
        mc = MongoClient(getattr(settings, 'MONGODB_CLIENT',
                                 'mongodb://localhost:27017/'))
        dbs = mc[database_name]
        dbs.drop_collection(collection_name)
        # print "success"
        return ""
    except:
        # error connecting to mongodb
        return str(sys.exc_info())


def mongodb_drop_database(database_name):
    """Drop Database"""
    try:
        mc = MongoClient(getattr(settings, 'MONGODB_CLIENT',
                                 'mongodb://localhost:27017/'))
        mc.drop_database(database_name)
        return ""

    except:
        # error connecting to mongodb
        # print str(sys.exc_info())
        return str(sys.exc_info())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

import json
from django.contrib.auth import login
from datetime import date


def authorize(request):
    a = HttpBasicAuthentication()
    if a.is_authenticated(request):
        login(request, request.user)
        auth = True
    else:
        if request.user.is_authenticated():
            auth = True
        else:
            auth = False
    return auth


def unauthorized_json_response(additional_info=None):
    body = {
        "code": 401,
        "message": "Unauthorized - Your account credentials were invalid.",
        "errors": [
            "Unauthorized - Your account credentials were invalid.",
        ]}
    if additional_info:
        body['message'] = "%s %s" % (body['message'], additional_info)
    body = json.dumps(body, indent=4, )
    return body


def json_response_400(error=""):
    body = {"code": 400,
            "message": "Client Error",
            "errors": [error, ]
            }
    body = json.dumps(body, indent=4, )
    return body


def json_response_404(error=""):
    body = {"code": 404,
            "message": "Not Found",
            "errors": [error, ]
            }
    body = json.dumps(body, indent=4, )
    return body


def json_response_500(error=""):
    body = {"code": 500,
            "message": "Server Side Error",
            "errors": [error, ]
            }
    body = json.dumps(body, indent=4, )
    return body



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.contrib import admin
from .models import Permission, SocialGraph

admin.site.register(Permission)
admin.site.register(SocialGraph)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django.contrib.auth.decorators import login_required
from django.views.generic.base import TemplateView
from django.views.decorators.csrf import csrf_exempt
from ..decorators import httpauth_login_required
from django.conf.urls import url
from .views import (previous_data_imports, delete_import,
                    import_data_file)

urlpatterns = [
    url(r'^view$', login_required(previous_data_imports),
        name="djmongo_previous_data_imports"),

    url(r'^view/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(previous_data_imports),
        name="djmongo_previous_data_imports_w_params"),

    url(r'^delete/(?P<slug>\S+)$',
        login_required(delete_import),
        name="djmongo_delete_import_by_slug"),

    url(r'^csv$', login_required(import_data_file),
        name="djmongo_import_csv"),

    url(r'^csv/(?P<database_name>[^/]+)$',
        login_required(import_data_file),
        name="djmongo_import_csv_w_db"),

    url(r'^csv/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        login_required(import_data_file),
        name="djmongo_import_csv_w_params"),

    # API calls ----------------------------------------------------------
    url(r'^api/csv$', httpauth_login_required(csrf_exempt(import_data_file)),
        name="djmongo_api_import_data_file"),

    url(r'^api/csv/(?P<database_name>[^/]+)/(?P<collection_name>[^/]+)$',
        httpauth_login_required(csrf_exempt(import_data_file)),
        name="djmongo_api_import_file_w_params"),

    url(r'^$',
        TemplateView.as_view(template_name='djmongo/console/import-home.html'),
        name="djmongo_import_home"),

]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.contrib import admin
from .models import (HTTPAuthDeleteAPI, IPAuthDeleteAPI,
                     OAuth2DeleteAPI, PublicDeleteAPI)


class HTTPAuthDeleteAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name',
                    'url')


admin.site.register(HTTPAuthDeleteAPI, HTTPAuthDeleteAPIAdmin)


class IPAuthDeleteAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name',
                    'url')


admin.site.register(IPAuthDeleteAPI, IPAuthDeleteAPIAdmin)


class PublicDeleteAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name', 'url',)


admin.site.register(PublicDeleteAPI, PublicDeleteAPIAdmin)


class OAuth2DeleteAPIAdmin(admin.ModelAdmin):
    list_display = ('slug', 'collection_name', 'database_name', 'url',)


admin.site.register(OAuth2DeleteAPI, OAuth2DeleteAPIAdmin)



from .models import WriteAPIOAuth2
from .views import write_to_collection

@csrf_exempt
def write_to_collection_oauth2(request, slug):
    return write_to_collection(request, slug, WriteAPIOAuth2)    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django import forms
from .models import (HTTPAuthDeleteAPI, PublicDeleteAPI,
                     IPAuthDeleteAPI, OAuth2DeleteAPI)


class OAuth2DeleteAPIForm(forms.ModelForm):

    class Meta:
        model = OAuth2DeleteAPI
        fields = ('slug', 'database_name', 'collection_name',
                  'search_keys', 'scopes', 'readme_md',)
    required_css_class = 'required'


class HTTPAuthDeleteAPIForm(forms.ModelForm):

    class Meta:
        model = HTTPAuthDeleteAPI
        fields = ('slug', 'database_name', 'collection_name',
                  'search_keys', 'groups', 'readme_md',)
    required_css_class = 'required'


class IPAuthDeleteAPIForm(forms.ModelForm):

    class Meta:
        model = IPAuthDeleteAPI
        fields = ('slug', 'database_name', 'collection_name',
                  'search_keys', 'from_ip', 'readme_md',)
    required_css_class = 'required'


class PublicDeleteAPIForm(forms.ModelForm):

    class Meta:
        model = PublicDeleteAPI
        fields = ('slug', 'database_name', 'collection_name',
                  'search_keys', 'readme_md',)
    required_css_class = 'required'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4

from django import forms
from .models import (HTTPAuthReadAPI, PublicReadAPI, IPAuthReadAPI,
                     CustomHTTPAuthReadAPI, CustomPublicReadAPI,
                     CustomIPAuthReadAPI, CustomOAuth2ReadAPI,
                     OAuth2ReadAPI)


class OAuth2ReadAPIForm(forms.ModelForm):

    class Meta:
        model = OAuth2ReadAPI
        fields = ('slug', 'database_name', 'collection_name',
                  'search_keys', 'scopes', 'readme_md',)
    required_css_class = 'required'


class HTTPAuthReadAPIForm(forms.ModelForm):

    class Meta:
        model = HTTPAuthReadAPI
        fields = ('slug', 'database_name', 'collection_name',
                  'search_keys', 'groups', 'readme_md',)
    required_css_class = 'required'


class IPAuthReadAPIForm(forms.ModelForm):

    class Meta:
        model = IPAuthReadAPI
        fields = ('slug', 'database_name', 'collection_name',
                  'search_keys', 'from_ip', 'readme_md',)
    required_css_class = 'required'


class PublicReadAPIForm(forms.ModelForm):

    class Meta:
        model = PublicReadAPI
        fields = ('slug', 'database_name', 'collection_name',
                  'search_keys', 'readme_md',)
    required_css_class = 'required'


class CustomHTTPAuthReadAPIForm(forms.ModelForm):

    class Meta:
        model = CustomHTTPAuthReadAPI
        fields = ('slug', 'query', 'group', 'type_mapper',
                  'return_keys', 'sort', 'default_limit',
                  'database_name', 'collection_name', 'output_format', 'readme_md',)

    required_css_class = 'required'


class CustomOAuth2ReadAPIForm(forms.ModelForm):

    class Meta:
        model = CustomOAuth2ReadAPI
        fields = ('slug', 'query', 'scopes', 'type_mapper',
                  'return_keys', 'sort', 'default_limit',
                  'database_name', 'collection_name', 'output_format', 'readme_md',)

    required_css_class = 'required'


class CustomPublicReadAPIForm(forms.ModelForm):

    class Meta:
        model = CustomPublicReadAPI
        fields = ('slug', 'query', 'type_mapper',
                  'return_keys', 'sort', 'default_limit',
                  'database_name', 'collection_name', 'output_format', 'readme_md',)

    required_css_class = 'required'


class CustomIPAuthReadAPIForm(forms.ModelForm):

    class Meta:
        model = CustomIPAuthReadAPI
        fields = ('slug', 'query', 'type_mapper',
                  'return_keys', 'sort', 'default_limit',
                  'database_name', 'collection_name',
                  'from_ip', 'output_format', 'readme_md',)

    required_css_class = 'required'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4
from django.conf.urls import url
from .views import write_to_collection_httpauth
from .oauth_views import write_to_collection_oauth2
from .urls import urlpatterns
from apps.dot_ext.decorators import capability_protected_resource

urlpatterns += [

    # Call / Run the write API
    url(r'^api/oauth2/(?P<slug>[^/]+)$',
        capability_protected_resource()(write_to_collection_oauth2),
        name="djmongo_api_write_to_collection_with_oauth2"),
    
]
